@@ -554,31 +554,31 @@ void bind_torrent_handle()
         .def("unset_flags", _(&torrent_handle::unset_flags))
         // deprecated
 #if TORRENT_ABI_VERSION == 1
-        .def("piece_priorities", &piece_priorities)
-        .def("file_priorities", &file_priorities)
-        .def("stop_when_ready", _(&torrent_handle::stop_when_ready))
-        .def("super_seeding", super_seeding1)
-        .def("auto_managed", _(&torrent_handle::auto_managed))
-        .def("set_priority", _(&torrent_handle::set_priority))
-        .def("get_torrent_info", &get_torrent_info)
-        .def("super_seeding", super_seeding0)
-        .def("write_resume_data", _(&torrent_handle::write_resume_data))
-        .def("is_seed", _(&torrent_handle::is_seed))
-        .def("is_finished", _(&torrent_handle::is_finished))
-        .def("has_metadata", _(&torrent_handle::has_metadata))
-        .def("use_interface", &torrent_handle::use_interface)
-        .def("name", _(&torrent_handle::name))
-        .def("is_paused", _(&torrent_handle::is_paused))
-        .def("is_auto_managed", _(&torrent_handle::is_auto_managed))
-        .def("set_upload_mode", _(&torrent_handle::set_upload_mode))
-        .def("set_share_mode", _(&torrent_handle::set_share_mode))
-        .def("apply_ip_filter", &torrent_handle::apply_ip_filter)
-        .def("set_sequential_download", _(&torrent_handle::set_sequential_download))
-        .def("set_peer_upload_limit", &torrent_handle::set_peer_upload_limit)
-        .def("set_peer_download_limit", &torrent_handle::set_peer_download_limit)
-        .def("set_ratio", _(&torrent_handle::set_ratio))
-        .def("save_path", _(&torrent_handle::save_path))
-        .def("set_tracker_login", &torrent_handle::set_tracker_login)
+        .def("piece_priorities", depr(&piece_priorities))
+        .def("file_priorities", depr(&file_priorities))
+        .def("stop_when_ready", depr(&torrent_handle::stop_when_ready))
+        .def("super_seeding", depr(super_seeding1))
+        .def("auto_managed", depr(&torrent_handle::auto_managed))
+        .def("set_priority", depr(&torrent_handle::set_priority))
+        .def("get_torrent_info", depr(&get_torrent_info))
+        .def("super_seeding", depr(super_seeding0))
+        .def("write_resume_data", depr(&torrent_handle::write_resume_data))
+        .def("is_seed", depr(&torrent_handle::is_seed))
+        .def("is_finished", depr(&torrent_handle::is_finished))
+        .def("has_metadata", depr(&torrent_handle::has_metadata))
+        .def("use_interface", depr(&torrent_handle::use_interface))
+        .def("name", depr(&torrent_handle::name))
+        .def("is_paused", depr(&torrent_handle::is_paused))
+        .def("is_auto_managed", depr(&torrent_handle::is_auto_managed))
+        .def("set_upload_mode", depr(&torrent_handle::set_upload_mode))
+        .def("set_share_mode", depr(&torrent_handle::set_share_mode))
+        .def("apply_ip_filter", depr(&torrent_handle::apply_ip_filter))
+        .def("set_sequential_download", depr(&torrent_handle::set_sequential_download))
+        .def("set_peer_upload_limit", depr(&torrent_handle::set_peer_upload_limit))
+        .def("set_peer_download_limit", depr(&torrent_handle::set_peer_download_limit))
+        .def("set_ratio", depr(&torrent_handle::set_ratio))
+        .def("save_path", depr(&torrent_handle::save_path))
+        .def("set_tracker_login", depr(&torrent_handle::set_tracker_login))
 #endif
         ;
 
