@@ -13,6 +13,7 @@
 #include "libtorrent/announce_entry.hpp"
 #include "libtorrent/tracker_manager.hpp" // for event_t
 #include "bytes.hpp"
+#include "gil.hpp"
 
 #ifdef _MSC_VER
 #pragma warning(push)
@@ -140,51 +141,124 @@ namespace
 #if TORRENT_ABI_VERSION == 1
     // Create getters for announce_entry data members with non-trivial types which need converting.
     lt::time_point get_next_announce(announce_entry const& ae)
-    { return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().info_hashes[protocol_version::V1].next_announce); }
+    {
+        python_deprecated("next_announce is deprecated");
+        return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().info_hashes[protocol_version::V1].next_announce);
+    }
     lt::time_point get_min_announce(announce_entry const& ae)
-    { return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().info_hashes[protocol_version::V1].min_announce); }
+    {
+        python_deprecated("min_announce is deprecated");
+        return ae.endpoints.empty() ? lt::time_point() : lt::time_point(ae.endpoints.front().info_hashes[protocol_version::V1].min_announce);
+    }
     // announce_entry data member bit-fields.
     int get_fails(announce_entry const& ae)
-    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].fails; }
+    {
+        python_deprecated("fails is deprecated");
+        return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].fails;
+    }
     bool get_updating(announce_entry const& ae)
-    { return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].updating; }
+    {
+        python_deprecated("updating is deprecated");
+        return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].updating;
+    }
     bool get_start_sent(announce_entry const& ae)
-    { return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].start_sent; }
+    {
+        python_deprecated("start_sent is deprecated");
+        return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].start_sent;
+    }
     bool get_complete_sent(announce_entry const& ae)
-    { return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].complete_sent; }
+    {
+        python_deprecated("complete_sent is deprecated");
+        return ae.endpoints.empty() ? false : ae.endpoints.front().info_hashes[protocol_version::V1].complete_sent;
+    }
     // announce_entry method requires lt::time_point.
     bool can_announce(announce_entry const& ae, bool is_seed) {
+        python_deprecated("can_announce() is deprecated");
         // an entry without endpoints implies it has never been announced so it can be now
         if (ae.endpoints.empty()) return true;
         lt::time_point now = lt::clock_type::now();
         return ae.endpoints.front().can_announce(now, is_seed, ae.fail_limit);
     }
     bool is_working(announce_entry const& ae)
-    { return ae.endpoints.empty() ? false : ae.endpoints.front().is_working(); }
+    {
+        python_deprecated("is_working is deprecated");
+        return ae.endpoints.empty() ? false : ae.endpoints.front().is_working();
+    }
 #endif
     int get_source(announce_entry const& ae) { return ae.source; }
     bool get_verified(announce_entry const& ae) { return ae.verified; }
 
 #if TORRENT_ABI_VERSION == 1
     std::string get_message(announce_entry const& ae)
-    { return ae.endpoints.empty() ? "" : ae.endpoints.front().info_hashes[protocol_version::V1].message; }
+    {
+        python_deprecated("message is deprecated");
+        return ae.endpoints.empty() ? "" : ae.endpoints.front().info_hashes[protocol_version::V1].message;
+    }
     error_code get_last_error(announce_entry const& ae)
-    { return ae.endpoints.empty() ? error_code() : ae.endpoints.front().info_hashes[protocol_version::V1].last_error; }
+    {
+        python_deprecated("last_error is deprecated");
+        return ae.endpoints.empty() ? error_code() : ae.endpoints.front().info_hashes[protocol_version::V1].last_error;
+    }
     int get_scrape_incomplete(announce_entry const& ae)
-    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_incomplete; }
+    {
+        python_deprecated("scrape_incomplete is deprecated");
+        return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_incomplete;
+    }
     int get_scrape_complete(announce_entry const& ae)
-    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_complete; }
+    {
+        python_deprecated("scrape_complete is deprecated");
+        return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_complete;
+    }
     int get_scrape_downloaded(announce_entry const& ae)
-    { return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_downloaded; }
-    int next_announce_in(announce_entry const&) { return 0; }
-    int min_announce_in(announce_entry const&) { return 0; }
-    bool get_send_stats(announce_entry const& ae) { return ae.send_stats; }
-    std::int64_t get_size(file_entry const& fe) { return fe.size; }
-    std::int64_t get_offset(file_entry const& fe) { return fe.offset; }
-    bool get_pad_file(file_entry const& fe) { return fe.pad_file; }
-    bool get_executable_attribute(file_entry const& fe) { return fe.executable_attribute; }
-    bool get_hidden_attribute(file_entry const& fe) { return fe.hidden_attribute; }
-    bool get_symlink_attribute(file_entry const& fe) { return fe.symlink_attribute; }
+    {
+        python_deprecated("scrape_downloaded is deprecated");
+        return ae.endpoints.empty() ? 0 : ae.endpoints.front().info_hashes[protocol_version::V1].scrape_downloaded;
+    }
+    int next_announce_in(announce_entry const&)
+    {
+        python_deprecated("next_announce_in is deprecated");
+        return 0;
+    }
+    int min_announce_in(announce_entry const&)
+    {
+        python_deprecated("min_announce_in is deprecated");
+        return 0;
+    }
+    bool get_send_stats(announce_entry const& ae)
+    {
+        python_deprecated("send_stats is deprecated");
+        return ae.send_stats;
+    }
+    std::int64_t get_size(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.size;
+    }
+    std::int64_t get_offset(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.offset;
+    }
+    bool get_pad_file(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.pad_file;
+    }
+    bool get_executable_attribute(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.executable_attribute;
+    }
+    bool get_hidden_attribute(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.hidden_attribute;
+    }
+    bool get_symlink_attribute(file_entry const& fe)
+    {
+        python_deprecated("file_entry is deprecated");
+        return fe.symlink_attribute;
+    }
 #endif
 
 } // namespace unnamed
@@ -237,32 +311,37 @@ std::shared_ptr<torrent_info> buffer_constructor1(bytes b, dict limits)
    return ret;
 }
 
-std::shared_ptr<torrent_info> file_constructor0(std::string const& filename)
+std::shared_ptr<torrent_info> file_constructor0(lt::string_view filename)
 {
-   return std::make_shared<torrent_info>(filename);
+   return std::make_shared<torrent_info>(std::string(filename));
 }
 
-std::shared_ptr<torrent_info> file_constructor1(std::string const& filename, dict limits)
+std::shared_ptr<torrent_info> file_constructor1(lt::string_view filename, dict limits)
 {
-   return std::make_shared<torrent_info>(filename, dict_to_limits(limits));
+   return std::make_shared<torrent_info>(std::string(filename), dict_to_limits(limits));
 }
 
-#if TORRENT_ABI_VERSION == 1
 std::shared_ptr<torrent_info> sha1_constructor0(sha1_hash const& ih)
 {
-   return std::make_shared<torrent_info>(info_hash_t(ih));
+    return std::make_shared<torrent_info>(info_hash_t(ih));
 }
-#endif
 
-std::shared_ptr<torrent_info> bencoded_constructor0(entry const& ent)
+std::shared_ptr<torrent_info> sha256_constructor0(sha256_hash const& ih)
 {
+    return std::make_shared<torrent_info>(info_hash_t(ih));
+}
+
+std::shared_ptr<torrent_info> bencoded_constructor0(dict d)
+{
+    entry ent = extract<entry>(d);
     std::vector<char> buf;
     bencode(std::back_inserter(buf), ent);
     return std::make_shared<torrent_info>(buf, lt::from_span);
 }
 
-std::shared_ptr<torrent_info> bencoded_constructor1(entry const& ent, dict limits)
+std::shared_ptr<torrent_info> bencoded_constructor1(dict d, dict limits)
 {
+    entry ent = extract<entry>(d);
     std::vector<char> buf;
     bencode(std::back_inserter(buf), ent);
     return std::make_shared<torrent_info>(buf, dict_to_limits(limits)
@@ -287,22 +366,6 @@ void bind_torrent_info()
         .value("V2", protocol_version::V2)
         ;
 
-    class_<info_hash_t>("info_hash_t")
-        .def(init<sha1_hash const&>(arg("sha1_hash")))
-        .def(init<sha256_hash const&>(arg("sha256_hash")))
-        .def(init<sha1_hash const&, sha256_hash const&>((arg("sha1_hash"), arg("sha256_hash"))))
-        .def("has_v1", &info_hash_t::has_v1)
-        .def("has_v2", &info_hash_t::has_v2)
-        .def("has", &info_hash_t::has)
-        .def("get", &info_hash_t::get)
-        .def("get_best", &info_hash_t::get_best)
-        .add_property("v1", &info_hash_t::v1)
-        .add_property("v2", &info_hash_t::v2)
-        .def(self == self)
-        .def(self != self)
-        .def(self < self)
-        ;
-
     enum_<announce_entry::tracker_source>("tracker_source")
         .value("source_torrent", announce_entry::source_torrent)
         .value("source_client", announce_entry::source_client)
@@ -322,13 +385,18 @@ void bind_torrent_info()
         .def("__init__", make_constructor(&file_constructor1))
         .def(init<torrent_info const&>((arg("ti"))))
 
-#if TORRENT_ABI_VERSION == 1
         .def("__init__", make_constructor(&sha1_constructor0))
-#endif
+        .def("__init__", make_constructor(&sha256_constructor0))
 
-        .def("add_tracker", (add_tracker1)&torrent_info::add_tracker, arg("url"), arg("tier") = 0, arg("source") = announce_entry::source_client)
-        .def("add_url_seed", &torrent_info::add_url_seed)
-        .def("add_http_seed", &torrent_info::add_http_seed)
+        .def("add_tracker", (add_tracker1)&torrent_info::add_tracker
+            , (arg("url"), arg("tier") = 0
+            , arg("source") = announce_entry::source_client))
+        .def("add_url_seed", &torrent_info::add_url_seed, (arg("url")
+            , arg("extern_auth") = std::string{}
+            , arg("extra_headers") = web_seed_entry::headers_t{}))
+        .def("add_http_seed", &torrent_info::add_http_seed, (arg("url")
+            , arg("extern_auth") = std::string{}
+            , arg("extra_headers") = web_seed_entry::headers_t{}))
         .def("web_seeds", get_web_seeds)
         .def("set_web_seeds", set_web_seeds)
 
@@ -342,8 +410,8 @@ void bind_torrent_info()
         .def("info_hashes", &torrent_info::info_hashes, copy)
         .def("hash_for_piece", &hash_for_piece)
 #if TORRENT_ABI_VERSION <= 2
-        .def("merkle_tree", get_merkle_tree)
-        .def("set_merkle_tree", set_merkle_tree)
+        .def("merkle_tree", depr(&get_merkle_tree))
+        .def("set_merkle_tree", depr(&set_merkle_tree))
 #endif
         .def("piece_size", &torrent_info::piece_size)
 
@@ -356,15 +424,14 @@ void bind_torrent_info()
         .def("files", &torrent_info::files, return_internal_reference<>())
         .def("orig_files", &torrent_info::orig_files, return_internal_reference<>())
 #if TORRENT_ABI_VERSION == 1
-        .def("file_at", &torrent_info::file_at)
-        .def("file_at_offset", &torrent_info::file_at_offset)
+        .def("file_at", depr(&torrent_info::file_at))
 #endif // TORRENT_ABI_VERSION
 
         .def("is_valid", &torrent_info::is_valid)
         .def("priv", &torrent_info::priv)
         .def("is_i2p", &torrent_info::is_i2p)
 #if TORRENT_ABI_VERSION <= 2
-        .def("is_merkle_torrent", &torrent_info::is_merkle_torrent)
+        .def("is_merkle_torrent", depr(&torrent_info::is_merkle_torrent))
 #endif
         .def("trackers", range(begin_trackers, end_trackers))
 
@@ -373,8 +440,8 @@ void bind_torrent_info()
         .def("add_node", &add_node)
         .def("nodes", &nodes)
 #if TORRENT_ABI_VERSION <= 2
-        .def("metadata", &metadata)
-        .def("metadata_size", &torrent_info::metadata_size)
+        .def("metadata", depr(&metadata))
+        .def("metadata_size", depr(&torrent_info::metadata_size))
 #endif
         .def("info_section", &get_info_section)
         .def("map_block", map_block)
@@ -418,14 +485,14 @@ void bind_torrent_info()
         .add_property("start_sent", &get_start_sent)
         .add_property("complete_sent", &get_complete_sent)
         .add_property("send_stats", &get_send_stats)
-        .def("next_announce_in", &next_announce_in)
-        .def("min_announce_in", &min_announce_in)
-        .def("can_announce", &can_announce)
-        .def("is_working", &is_working)
+        .def("next_announce_in", depr(&next_announce_in))
+        .def("min_announce_in", depr(&min_announce_in))
+        .def("can_announce", depr(&can_announce))
+        .def("is_working", depr(&is_working))
 #endif
 #if TORRENT_ABI_VERSION <= 2
-        .def("reset", &announce_entry::reset)
-        .def("trim", &announce_entry::trim)
+        .def("reset", depr(&announce_entry::reset))
+        .def("trim", depr(&announce_entry::trim))
 #endif
         ;
 
