@@ -49,15 +49,15 @@ public:
         , m_next_layer(std::forward<Arg>(arg))
         , m_tls_version(ctx.m_impl->tls_version())
     {
-        set_impl(ctx.m_impl->is_server() ? server : client);
+        ensure_impl(ctx.m_impl->is_server() ? server : client);
     }
 
     stream(stream&& other)
         : stream_base(std::move(other))
-        , m_tls_version(std::move(other.m_tls_version))
         , m_next_layer(std::move(other.m_next_layer))
         , m_verify(std::move(other.m_verify))
         , m_verify_callback(std::move(other.m_verify_callback))
+        , m_tls_version(other.m_tls_version)
         , m_impl(std::move(other.m_impl))
     {
         m_impl->parent = this;
@@ -66,7 +66,10 @@ public:
     stream(stream const& other) = delete;
     ~stream()
     {
-        if (m_impl) m_impl->parent = nullptr;
+        if (m_impl) {
+            m_impl->abort();
+            m_impl->parent = nullptr;
+        }
     }
 
     executor_type get_executor() { return m_next_layer.get_executor(); }
@@ -123,12 +126,18 @@ public:
     }
 
     template <typename HandshakeHandler>
-    void async_handshake(handshake_type type, HandshakeHandler handler)
+    BOOST_ASIO_INITFN_RESULT_TYPE(HandshakeHandler, void(error_code))
+    async_handshake(handshake_type type, HandshakeHandler&& handler)
     {
+        // If you get an error on the following line it means that your handler does
+        // not meet the documented type requirements for a HandshakeHandler.
+        BOOST_ASIO_HANDSHAKE_HANDLER_CHECK(HandshakeHandler, handler) type_check;
+
+        async_callable<HandshakeHandler, error_code> callable(std::move(handler));
+
         if (m_impl->handshake_handler || m_impl->is_handshake_done)
         {
-            post(get_executor(),
-                 std::bind(std::move(handler), boost::asio::error::operation_not_supported));
+            post(get_executor(), std::bind(callable, boost::asio::error::operation_not_supported));
             return;
         }
 
@@ -136,32 +145,46 @@ public:
         m_next_layer.non_blocking(true, ec);
         if (ec)
         {
-            post(get_executor(), std::bind(std::move(handler), ec));
+            post(get_executor(), std::bind(callable, ec));
             return;
         }
 
-        set_impl(type);
-        m_impl->handshake_handler = std::move(handler);
+        ensure_impl(type);
+        m_impl->handshake_handler = std::bind(callable, std::placeholders::_1);
         m_impl->handle_handshake();
+        return callable.get_completion_result();
     }
 
     template <typename ConstBufferSequence, typename BufferedHandshakeHandler>
-    void async_handshake(handshake_type type,
-                         const ConstBufferSequence& buffers,
-                         BufferedHandshakeHandler handler)
+    BOOST_ASIO_INITFN_RESULT_TYPE(BufferedHandshakeHandler, void(error_code, std::size_t))
+    async_handshake(handshake_type type,
+                    const ConstBufferSequence& buffers,
+                    BufferedHandshakeHandler&& handler)
     {
-        async_handshake(type, std::bind(handler, std::placeholders::_1, std::size_t(0)));
+        // If you get an error on the following line it means that your handler does
+        // not meet the documented type requirements for a BufferedHandshakeHandler.
+        BOOST_ASIO_BUFFERED_HANDSHAKE_HANDLER_CHECK(BufferedHandshakeHandler, handler) type_check;
+
+        async_callable<BufferedHandshakeHandler, error_code, std::size_t> callable(
+            std::move(handler));
+
+        async_handshake(type, std::bind(callable, std::placeholders::_1, std::size_t(0)));
+        return callable.get_completion_result();
     }
 
-    template <typename MutableBufferSequence, typename ReadHandler>
-    void async_read_some(const MutableBufferSequence& buffers, ReadHandler handler)
+    template <typename ShutdownHandler>
+    BOOST_ASIO_INITFN_RESULT_TYPE(ShutdownHandler, void(error_code))
+    async_shutdown(ShutdownHandler&& handler)
     {
-        if (m_impl->read_handler)
+        // If you get an error on the following line it means that your handler does
+        // not meet the documented type requirements for a ShutdownHandler.
+        BOOST_ASIO_SHUTDOWN_HANDLER_CHECK(ShutdownHandler, handler) type_check;
+
+        async_callable<ShutdownHandler, error_code> callable(std::move(handler));
+
+        if (m_impl->shutdown_handler || !m_impl->is_handshake_done)
         {
-            post(get_executor(),
-                 std::bind(std::move(handler),
-                           boost::asio::error::operation_not_supported,
-                           std::size_t(0)));
+            post(get_executor(), std::bind(callable, boost::asio::error::operation_not_supported));
             return;
         }
 
@@ -169,7 +192,38 @@ public:
         m_next_layer.non_blocking(true, ec);
         if (ec)
         {
-            post(get_executor(), std::bind(std::move(handler), ec, std::size_t(0)));
+            post(get_executor(), std::bind(callable, ec));
+            return;
+        }
+
+        m_impl->abort();
+        m_impl->shutdown_handler = std::bind(callable, std::placeholders::_1);
+        m_impl->handle_shutdown();
+        return callable.get_completion_result();
+    }
+
+    template <typename MutableBufferSequence, typename ReadHandler>
+    BOOST_ASIO_INITFN_RESULT_TYPE(ReadHandler, void(error_code, std::size_t))
+    async_read_some(const MutableBufferSequence& buffers, ReadHandler&& handler)
+    {
+        // If you get an error on the following line it means that your handler does
+        // not meet the documented type requirements for a ReadHandler.
+        BOOST_ASIO_READ_HANDLER_CHECK(ReadHandler, handler) type_check;
+
+        async_callable<ReadHandler, error_code, std::size_t> callable(std::move(handler));
+
+        if (m_impl->read_handler)
+        {
+            post(get_executor(),
+                 std::bind(callable, boost::asio::error::operation_not_supported, std::size_t(0)));
+            return;
+        }
+
+        error_code ec;
+        m_next_layer.non_blocking(true, ec);
+        if (ec)
+        {
+            post(get_executor(), std::bind(callable, ec, std::size_t(0)));
             return;
         }
 
@@ -177,31 +231,40 @@ public:
         for (auto b = buffer_sequence_begin(buffers), end(buffer_sequence_end(buffers)); b != end;
              ++b)
         {
-            if (b->size() == 0) continue;
-            m_impl->read_buffers.push_back(*b);
-            bytes_added += b->size();
+            auto r = *b; // operator -> might be deleted
+            if (r.size() == 0) continue;
+            m_impl->read_buffers.push_back(r);
+            bytes_added += r.size();
         }
 
         if (bytes_added == 0)
         {
             // if we're reading 0 bytes, post handler immediately
-            post(get_executor(), std::bind<void>(std::move(handler), error_code(), std::size_t(0)));
+            post(get_executor(), std::bind(callable, error_code(), std::size_t(0)));
             return;
         }
 
-        m_impl->read_handler = std::move(handler);
+        using namespace std::placeholders;
+        m_impl->read_handler = std::bind(callable, _1, _2);
+        m_impl->bytes_read = 0;
         m_impl->async_schedule();
+        return callable.get_completion_result();
     }
 
     template <typename ConstBufferSequence, typename WriteHandler>
-    void async_write_some(const ConstBufferSequence& buffers, WriteHandler handler)
+    BOOST_ASIO_INITFN_RESULT_TYPE(WriteHandler, void(error_code, std::size_t))
+    async_write_some(const ConstBufferSequence& buffers, WriteHandler&& handler)
     {
+        // If you get an error on the following line it means that your handler does
+        // not meet the documented type requirements for a WriteHandler.
+        BOOST_ASIO_WRITE_HANDLER_CHECK(WriteHandler, handler) type_check;
+
+        async_callable<WriteHandler, error_code, std::size_t> callable(std::move(handler));
+
         if (m_impl->write_handler)
         {
             post(get_executor(),
-                 std::bind(std::move(handler),
-                           boost::asio::error::operation_not_supported,
-                           std::size_t(0)));
+                 std::bind(callable, boost::asio::error::operation_not_supported, std::size_t(0)));
             return;
         }
 
@@ -209,7 +272,7 @@ public:
         m_next_layer.non_blocking(true, ec);
         if (ec)
         {
-            post(get_executor(), std::bind(std::move(handler), ec, std::size_t(0)));
+            post(get_executor(), std::bind(callable, ec, std::size_t(0)));
             return;
         }
 
@@ -217,42 +280,24 @@ public:
         for (auto b = buffer_sequence_begin(buffers), end(buffer_sequence_end(buffers)); b != end;
              ++b)
         {
-            if (b->size() == 0) continue;
-            m_impl->write_buffers.push_back(*b);
-            bytes_added += b->size();
+            auto r = *b; // operator -> might be deleted
+            if (r.size() == 0) continue;
+            m_impl->write_buffers.push_back(r);
+            bytes_added += r.size();
         }
 
         if (bytes_added == 0)
         {
             // if we're writing 0 bytes, post handler immediately
-            post(get_executor(), std::bind<void>(std::move(handler), error_code(), std::size_t(0)));
+            post(get_executor(), std::bind<void>(callable, error_code(), std::size_t(0)));
             return;
         }
 
-        m_impl->write_handler = std::move(handler);
+        using namespace std::placeholders;
+        m_impl->write_handler = std::bind(callable, _1, _2);
         m_impl->bytes_written = 0;
         m_impl->async_schedule();
-    }
-
-    template <typename ShutdownHandler> void async_shutdown(ShutdownHandler handler)
-    {
-        if (m_impl->shutdown_handler || !m_impl->is_handshake_done)
-        {
-            post(get_executor(),
-                 std::bind(std::move(handler), boost::asio::error::operation_not_supported));
-            return;
-        }
-
-        error_code ec;
-        m_next_layer.non_blocking(true, ec);
-        if (ec)
-        {
-            post(get_executor(), std::bind(std::move(handler), ec));
-            return;
-        }
-
-        m_impl->shutdown_handler = std::move(handler);
-        m_impl->handle_shutdown();
+        return callable.get_completion_result();
     }
 
     void handshake(handshake_type type)
@@ -264,7 +309,10 @@ public:
 
     error_code handshake(handshake_type type, error_code& ec)
     {
-        set_impl(type);
+        if (m_impl->is_handshake_done)
+            return ec = boost::asio::error::operation_not_supported;
+
+        ensure_impl(type);
         int ret;
         do {
             ret = gnutls_handshake(m_impl->session);
@@ -317,7 +365,7 @@ public:
     template <typename MutableBufferSequence> size_t read_some(const MutableBufferSequence& buffers)
     {
         error_code ec;
-        auto bytes_read = read_some(buffers, ec);
+        std::size_t bytes_read = read_some(buffers, ec);
         if (ec) boost::throw_exception(boost::system::system_error(ec));
         return bytes_read;
     }
@@ -346,9 +394,9 @@ public:
     std::size_t write_some(const ConstBufferSequence& buffers)
     {
         error_code ec;
-        auto bytes_sent = write_some(buffers, ec);
+        std::size_t bytes_written = write_some(buffers, ec);
         if (ec) boost::throw_exception(boost::system::system_error(ec));
-        return bytes_sent;
+        return bytes_written;
     }
 #endif
 
@@ -392,6 +440,35 @@ public:
     // -----------------------------------
 
 private:
+    template <typename Handler, typename... Args> class async_callable
+    {
+    public:
+        async_callable(Handler&& h)
+            : m_impl(std::make_shared<impl>(std::move(h)))
+        {}
+
+        void operator()(Args... args) const
+        {
+            m_impl->completion.completion_handler(std::forward<Args>(args)...);
+        }
+
+        auto get_completion_result() { return m_impl->completion.result.get(); }
+
+    private:
+        struct impl
+        {
+            impl(Handler&& h)
+                : handler(std::move(h))
+                , completion(handler)
+            {}
+
+            Handler handler;
+            boost::asio::async_completion<Handler, void(Args...)> completion;
+        };
+
+        std::shared_ptr<impl> m_impl;
+    };
+
     enum class direction
     {
         none,
@@ -399,18 +476,10 @@ private:
         write
     };
 
-    void set_impl(handshake_type type)
-    {
-        if (!m_impl || m_impl->type != type)
-            if (auto old = std::exchange(m_impl, std::make_shared<impl>(this, type)))
-                old->parent = nullptr;
-    }
-
     next_layer_type m_next_layer;
     verify_mode m_verify = -1;
     std::function<bool(bool preverified, verify_context& ctx)> m_verify_callback;
-
-    const unsigned int m_tls_version; // X*10 + Y => TLS X.Y, 0*10 + Z => SSL Z
+    unsigned int m_tls_version; // X*10 + Y => TLS X.Y, 0*10 + Z => SSL Z
 
     struct impl : public std::enable_shared_from_this<impl>
     {
@@ -460,6 +529,23 @@ private:
 
         ~impl() { gnutls_deinit(session); }
 
+        template<typename Function> void post(Function&& function) const
+        {
+            if(parent)
+                boost::asio::post(parent->get_executor(), std::forward<Function>(function));
+        }
+
+        void abort() {
+            if(auto handler = std::exchange(handshake_handler, nullptr))
+                handler(boost::asio::error::operation_aborted);
+            if(auto handler = std::exchange(shutdown_handler, nullptr))
+                handler(boost::asio::error::operation_aborted);
+            if(auto handler = std::exchange(read_handler, nullptr))
+                handler(boost::asio::error::operation_aborted, std::size_t(0));
+            if(auto handler = std::exchange(write_handler, nullptr))
+                handler(boost::asio::error::operation_aborted, std::size_t(0));
+        }
+
         std::string get_server_name() const
         {
             char buf[256];
@@ -494,7 +580,7 @@ private:
             if (want_write() && !std::exchange(is_writing, true))
             {
                 next_layer.async_wait(next_layer_type::wait_write,
-                                      std::bind(&impl::handle_write, this->shared_from_this(), _1));
+                        std::bind(&impl::handle_write, this->shared_from_this(), _1));
             }
         }
 
@@ -511,7 +597,8 @@ private:
 
                 read_buffers.clear();
                 auto handler = std::exchange(read_handler, nullptr);
-                handler(ec, std::exchange(bytes_read, std::size_t(0)));
+                post(std::bind(std::move(handler), ec, std::exchange(bytes_read, std::size_t(0))));
+                return;
             }
 
             if (handshake_handler) return handle_handshake(ec);
@@ -531,7 +618,7 @@ private:
 
                 write_buffers.clear();
                 auto handler = std::exchange(write_handler, nullptr);
-                handler(ec, std::exchange(bytes_written, std::size_t(0)));
+                post(std::bind(std::move(handler), ec, std::exchange(bytes_written, std::size_t(0))));
             }
 
             if (handshake_handler) return handle_handshake(ec);
@@ -558,7 +645,7 @@ private:
             }
 
             auto handler = std::exchange(handshake_handler, nullptr);
-            handler(ec);
+            post(std::bind(std::move(handler), ec));
         }
 
         void handle_shutdown(error_code ec = {})
@@ -581,7 +668,7 @@ private:
             }
 
             auto handler = std::exchange(shutdown_handler, nullptr);
-            handler(ec);
+            post(std::bind(std::move(handler), ec));
         }
 
         std::size_t recv_some(error_code& ec)
@@ -810,6 +897,16 @@ private:
         std::size_t bytes_written = 0;
     };
 
+    std::shared_ptr<impl> ensure_impl(handshake_type type)
+    {
+        if (!m_impl || m_impl->type != type)
+            if (auto old = std::exchange(m_impl, std::make_shared<impl>(this, type))) {
+                old->abort();
+                old->parent = nullptr;
+            }
+        return m_impl;
+    }
+
     std::shared_ptr<impl> m_impl;
 };
 
