@@ -1,6 +1,12 @@
 /*
 
-Copyright (c) 2003-2017, Arvid Norberg
+Copyright (c) 2003-2020, Arvid Norberg
+Copyright (c) 2015, Mike Tzou
+Copyright (c) 2016, 2018-2019, Alden Torres
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2017, AllSeeingEyeTolledEweSew
+Copyright (c) 2017-2018, Steven Siloti
+Copyright (c) 2019, Pavel Pimenov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -56,6 +62,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/entry.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/session.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/identify_client.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/ip_filter.hpp"
@@ -68,33 +75,51 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/write_resume_data.hpp"
 #include "libtorrent/string_view.hpp"
 #include "libtorrent/disk_interface.hpp" // for open_file_state
+#include "libtorrent/disabled_disk_io.hpp" // for disabled_disk_io_constructor
 
 #include "torrent_view.hpp"
 #include "session_view.hpp"
 #include "print.hpp"
 
+
+#ifdef _WIN32
+
+#include <windows.h>
+#include <conio.h>
+
+#else
+
+#include <termios.h>
+#include <sys/ioctl.h>
+#include <csignal>
+#include <utility>
+#include <dirent.h>
+
+#endif
+
+namespace {
+
 using lt::total_milliseconds;
 using lt::alert;
 using lt::piece_index_t;
 using lt::file_index_t;
 using lt::torrent_handle;
 using lt::add_torrent_params;
-using lt::cache_status;
 using lt::total_seconds;
 using lt::torrent_flags_t;
 using lt::seconds;
 using lt::operator "" _sv;
 using lt::address_v4;
 using lt::address_v6;
+using lt::make_address_v6;
+using lt::make_address_v4;
+using lt::make_address;
 
 using std::chrono::duration_cast;
 using std::stoi;
 
 #ifdef _WIN32
 
-#include <windows.h>
-#include <conio.h>
-
 bool sleep_and_input(int* c, lt::time_duration const sleep)
 {
 	for (int i = 0; i < 2; ++i)
@@ -111,12 +136,6 @@ bool sleep_and_input(int* c, lt::time_duration const sleep)
 
 #else
 
-#include <termios.h>
-#include <sys/ioctl.h>
-#include <csignal>
-#include <utility>
-#include <dirent.h>
-
 struct set_keypress
 {
 	enum terminal_mode {
@@ -126,16 +145,18 @@ struct set_keypress
 
 	explicit set_keypress(std::uint8_t const mode = 0)
 	{
+		using ul = unsigned long;
+
 		termios new_settings;
 		tcgetattr(0, &stored_settings);
 		new_settings = stored_settings;
 		// Disable canonical mode, and set buffer size to 1 byte
 		// and disable echo
 		if (mode & echo) new_settings.c_lflag |= ECHO;
-		else new_settings.c_lflag &= ~ECHO;
+		else new_settings.c_lflag &= ul(~ECHO);
 
 		if (mode & canonical) new_settings.c_lflag |= ICANON;
-		else new_settings.c_lflag &= ~ICANON;
+		else new_settings.c_lflag &= ul(~ICANON);
 
 		new_settings.c_cc[VTIME] = 0;
 		new_settings.c_cc[VMIN] = 1;
@@ -154,8 +175,8 @@ retry:
 	fd_set set;
 	FD_ZERO(&set);
 	FD_SET(0, &set);
-	int const delay = total_milliseconds(done - lt::clock_type::now());
-	timeval tv = {delay / 1000, (delay % 1000) * 1000 };
+	auto const delay = total_milliseconds(done - lt::clock_type::now());
+	timeval tv = {int(delay / 1000), int((delay % 1000) * 1000) };
 	ret = select(1, &set, nullptr, nullptr, &tv);
 	if (ret > 0)
 	{
@@ -189,7 +210,6 @@ bool print_matrix = false;
 bool print_file_progress = false;
 bool show_pad_files = false;
 bool show_dht_status = false;
-bool sequential_download = false;
 
 bool print_ip = true;
 bool print_local_ip = false;
@@ -225,7 +245,7 @@ bool load_file(std::string const& filename, std::vector<char>& v
 	f.seekg(0, std::ios_base::beg);
 	v.resize(static_cast<std::size_t>(s));
 	if (s == std::fstream::pos_type(0)) return !f.fail();
-	f.read(v.data(), v.size());
+	f.read(v.data(), int(v.size()));
 	return !f.fail();
 }
 
@@ -235,7 +255,7 @@ bool is_absolute_path(std::string const& f)
 #if defined(TORRENT_WINDOWS) || defined(TORRENT_OS2)
 	int i = 0;
 	// match the xx:\ or xx:/ form
-	while (f[i] && strchr("abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ", f[i])) ++i;
+	while (f[i] && strchr("abcdefghijklmnopqrstuvxyz", f[i])) ++i;
 	if (i < int(f.size()-1) && f[i] == ':' && (f[i+1] == '\\' || f[i+1] == '/'))
 		return true;
 
@@ -283,13 +303,12 @@ std::string make_absolute_path(std::string const& p)
 std::string print_endpoint(lt::tcp::endpoint const& ep)
 {
 	using namespace lt;
-	lt::error_code ec;
 	char buf[200];
 	address const& addr = ep.address();
 	if (addr.is_v6())
-		std::snprintf(buf, sizeof(buf), "[%s]:%d", addr.to_string(ec).c_str(), ep.port());
+		std::snprintf(buf, sizeof(buf), "[%s]:%d", addr.to_string().c_str(), ep.port());
 	else
-		std::snprintf(buf, sizeof(buf), "%s:%d", addr.to_string(ec).c_str(), ep.port());
+		std::snprintf(buf, sizeof(buf), "%s:%d", addr.to_string().c_str(), ep.port());
 	return buf;
 }
 
@@ -355,7 +374,7 @@ int print_peer_info(std::string& out
 		temp[7] = 0;
 
 		char peer_progress[10];
-		std::snprintf(peer_progress, sizeof(peer_progress), "%.1f%%", i->progress_ppm / 10000.f);
+		std::snprintf(peer_progress, sizeof(peer_progress), "%.1f%%", i->progress_ppm / 10000.0);
 		std::snprintf(str, sizeof(str)
 			, "%s %s%s (%s|%s) %s%s (%s|%s) %s%7s %4d%4d%4d %s%s%s%s%s%s%s%s%s%s%s%s%s%s %s%s%s %s%s%s %s%s%s%s%s%s "
 			, progress_bar(i->progress_ppm / 1000, 15, col_green, '#', '-', peer_progress).c_str()
@@ -487,30 +506,31 @@ int poll_interval = 5;
 int max_connections_per_torrent = 50;
 bool seed_mode = false;
 bool stats_enabled = false;
-int cache_size = -1;
+bool exit_on_finish = false;
 
 bool share_mode = false;
-bool disable_storage = false;
 
 bool quit = false;
 
+#ifndef _WIN32
 void signal_handler(int)
 {
 	// make the main loop terminate
 	quit = true;
 }
+#endif
 
 // if non-empty, a peer that will be added to all torrents
 std::string peer;
 
 void print_settings(int const start, int const num
-	, char const* const fmt)
+	, char const* const type)
 {
 	for (int i = start; i < start + num; ++i)
 	{
 		char const* name = lt::name_for_setting(i);
 		if (!name || name[0] == '\0') continue;
-		std::printf(fmt, name);
+		std::printf("%s=<%s>\n", name, type);
 	}
 }
 
@@ -547,7 +567,7 @@ void assign_setting(lt::settings_pack& settings, std::string const& key, char co
 			}
 			break;
 		case settings_pack::int_type_base:
-			using namespace libtorrent::literals;
+			using namespace lt::literals;
 			static std::map<lt::string_view, int> const enums = {
 				{"no_piece_suggestions"_sv, settings_pack::no_piece_suggestions},
 				{"suggest_read_cache"_sv, settings_pack::suggest_read_cache},
@@ -575,11 +595,13 @@ void assign_setting(lt::settings_pack& settings, std::string const& key, char co
 				{"i2p_proxy"_sv, settings_pack::i2p_proxy},
 			};
 
-			auto const it = enums.find(lt::string_view(value));
-			if (it != enums.end())
 			{
-				settings.set_int(sett_name, it->second);
-				break;
+				auto const it = enums.find(lt::string_view(value));
+				if (it != enums.end())
+				{
+					settings.set_int(sett_name, it->second);
+					break;
+				}
 			}
 
 			static std::map<lt::string_view, lt::alert_category_t> const alert_categories = {
@@ -631,10 +653,10 @@ void assign_setting(lt::settings_pack& settings, std::string const& key, char co
 	}
 }
 
-std::string resume_file(lt::sha1_hash const& info_hash)
+std::string resume_file(lt::info_hash_t const& info_hash)
 {
 	return path_append(save_path, path_append(".resume"
-		, to_hex(info_hash) + ".resume"));
+		, to_hex(info_hash.get_best()) + ".resume"));
 }
 
 void set_torrent_params(lt::add_torrent_params& p)
@@ -645,7 +667,6 @@ void set_torrent_params(lt::add_torrent_params& p)
 	p.download_limit = torrent_download_limit;
 
 	if (seed_mode) p.flags |= lt::torrent_flags::seed_mode;
-	if (disable_storage) p.storage = lt::disabled_storage_constructor;
 	if (share_mode) p.flags |= lt::torrent_flags::share_mode;
 	p.save_path = save_path;
 	p.storage_mode = allocation_mode;
@@ -664,7 +685,7 @@ void add_magnet(lt::session& ses, lt::string_view uri)
 	}
 
 	std::vector<char> resume_data;
-	if (load_file(resume_file(p.info_hash), resume_data))
+	if (load_file(resume_file(p.info_hashes), resume_data))
 	{
 		p = lt::read_resume_data(resume_data, ec);
 		if (ec) std::printf("  failed to load resume data: %s\n", ec.message().c_str());
@@ -698,7 +719,7 @@ bool add_torrent(lt::session& ses, std::string torrent)
 	add_torrent_params p;
 
 	std::vector<char> resume_data;
-	if (load_file(resume_file(ti->info_hash()), resume_data))
+	if (load_file(resume_file(ti->info_hashes()), resume_data))
 	{
 		p = lt::read_resume_data(resume_data, ec);
 		if (ec) std::printf("  failed to load resume data: %s\n", ec.message().c_str());
@@ -818,14 +839,18 @@ void print_alert(lt::alert const* a, std::string& str)
 	str += a->message();
 	str += esc("0");
 
+	static auto const first_ts = a->timestamp();
+
 	if (g_log_file)
-		std::fprintf(g_log_file, "[%s] %s\n", timestamp(),  a->message().c_str());
+		std::fprintf(g_log_file, "[%" PRId64 "] %s\n"
+			, std::int64_t(duration_cast<std::chrono::milliseconds>(a->timestamp() - first_ts).count())
+			,  a->message().c_str());
 }
 
 int save_file(std::string const& filename, std::vector<char> const& v)
 {
 	std::fstream f(filename, std::ios_base::trunc | std::ios_base::out | std::ios_base::binary);
-	f.write(v.data(), v.size());
+	f.write(v.data(), int(v.size()));
 	return !f.fail();
 }
 
@@ -838,8 +863,7 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 
 	if (session_stats_alert* s = alert_cast<session_stats_alert>(a))
 	{
-		ses_view.update_counters(s->counters()
-			, duration_cast<microseconds>(s->timestamp().time_since_epoch()).count());
+		ses_view.update_counters(s->counters(), s->timestamp());
 		return !stats_enabled;
 	}
 
@@ -852,7 +876,7 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 	}
 #endif
 
-#ifdef TORRENT_USE_OPENSSL
+#ifdef TORRENT_SSL_PEERS
 	if (torrent_need_cert_alert* p = alert_cast<torrent_need_cert_alert>(a))
 	{
 		torrent_handle h = p->handle;
@@ -926,7 +950,8 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 		h.save_resume_data(torrent_handle::save_info_dict);
 		++num_outstanding_resume_data;
 	}
-	else if (add_torrent_alert* p = alert_cast<add_torrent_alert>(a))
+
+	if (add_torrent_alert* p = alert_cast<add_torrent_alert>(a))
 	{
 		if (p->error)
 		{
@@ -944,20 +969,21 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 			// if we have a peer specified, connect to it
 			if (!peer.empty())
 			{
-				char* port = (char*) strrchr((char*)peer.c_str(), ':');
-				if (port != nullptr)
+				auto port = peer.find_last_of(':');
+				if (port != std::string::npos)
 				{
-					*port++ = 0;
-					char const* ip = peer.c_str();
-					int peer_port = atoi(port);
+					peer[port++] = '\0';
+					char const* ip = peer.data();
+					int const peer_port = atoi(peer.data() + port);
 					error_code ec;
 					if (peer_port > 0)
-						h.connect_peer(tcp::endpoint(address::from_string(ip, ec), std::uint16_t(peer_port)));
+						h.connect_peer(tcp::endpoint(make_address(ip, ec), std::uint16_t(peer_port)));
 				}
 			}
 		}
 	}
-	else if (torrent_finished_alert* p = alert_cast<torrent_finished_alert>(a))
+
+	if (torrent_finished_alert* p = alert_cast<torrent_finished_alert>(a))
 	{
 		p->handle.set_max_connections(max_connections_per_torrent / 2);
 
@@ -967,21 +993,25 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 		torrent_handle h = p->handle;
 		h.save_resume_data(torrent_handle::save_info_dict);
 		++num_outstanding_resume_data;
+		if (exit_on_finish) quit = true;
 	}
-	else if (save_resume_data_alert* p = alert_cast<save_resume_data_alert>(a))
+
+	if (save_resume_data_alert* p = alert_cast<save_resume_data_alert>(a))
 	{
 		--num_outstanding_resume_data;
 		auto const buf = write_resume_data_buf(p->params);
-		save_file(resume_file(p->params.info_hash), buf);
+		save_file(resume_file(p->params.info_hashes), buf);
 	}
-	else if (save_resume_data_failed_alert* p = alert_cast<save_resume_data_failed_alert>(a))
+
+	if (save_resume_data_failed_alert* p = alert_cast<save_resume_data_failed_alert>(a))
 	{
 		--num_outstanding_resume_data;
 		// don't print the error if it was just that we didn't need to save resume
 		// data. Returning true means "handled" and not printed to the log
 		return p->error == lt::errors::resume_data_not_modified;
 	}
-	else if (torrent_paused_alert* p = alert_cast<torrent_paused_alert>(a))
+
+	if (torrent_paused_alert* p = alert_cast<torrent_paused_alert>(a))
 	{
 		// write resume data for the finished torrent
 		// the alert handler for save_resume_data_alert
@@ -990,12 +1020,14 @@ bool handle_alert(torrent_view& view, session_view& ses_view
 		h.save_resume_data(torrent_handle::save_info_dict);
 		++num_outstanding_resume_data;
 	}
-	else if (state_update_alert* p = alert_cast<state_update_alert>(a))
+
+	if (state_update_alert* p = alert_cast<state_update_alert>(a))
 	{
 		view.update_torrents(std::move(p->status));
 		return true;
 	}
-	else if (torrent_removed_alert* p = alert_cast<torrent_removed_alert>(a))
+
+	if (torrent_removed_alert* p = alert_cast<torrent_removed_alert>(a))
 	{
 		view.remove_torrent(std::move(p->handle));
 	}
@@ -1024,64 +1056,52 @@ void pop_alerts(torrent_view& view, session_view& ses_view
 	}
 }
 
-void print_piece(lt::partial_piece_info const* pp
-	, lt::cached_piece_info const* cs
+void print_piece(lt::partial_piece_info const& pp
 	, std::vector<lt::peer_info> const& peers
 	, std::string& out)
 {
 	using namespace lt;
 
 	char str[1024];
-	assert(pp == nullptr || cs == nullptr || cs->piece == pp->piece_index);
-	int piece = static_cast<int>(pp ? pp->piece_index : cs->piece);
-	int num_blocks = pp ? pp->blocks_in_piece : int(cs->blocks.size());
+	int const piece = static_cast<int>(pp.piece_index);
+	int const num_blocks = pp.blocks_in_piece;
 
 	std::snprintf(str, sizeof(str), "%5d:[", piece);
 	out += str;
 	string_view last_color;
 	for (int j = 0; j < num_blocks; ++j)
 	{
-		int const index = pp ? peer_index(pp->blocks[j].peer(), peers) % 36 : -1;
+		int const index = peer_index(pp.blocks[j].peer(), peers) % 36;
+		bool const snubbed = index >= 0 ? bool(peers[std::size_t(index)].flags & lt::peer_info::snubbed) : false;
 		char const* chr = " ";
-		bool const snubbed = index >= 0 ? bool(peers[index].flags & lt::peer_info::snubbed) : false;
-
 		char const* color = "";
 
-		if (pp == nullptr)
+		if (pp.blocks[j].bytes_progress > 0
+				&& pp.blocks[j].state == block_info::requested)
 		{
-			color = cs->blocks[j] ? esc("34;7") : esc("0");
-			chr = " ";
-		}
-		else
-		{
-			if (cs && cs->blocks[j] && pp->blocks[j].state != block_info::finished)
-				color = esc("36;7");
-			else if (pp->blocks[j].bytes_progress > 0
-					&& pp->blocks[j].state == block_info::requested)
-			{
-				if (pp->blocks[j].num_peers > 1) color = esc("0;1");
-				else color = snubbed ? esc("0;35") : esc("0;33");
+			if (pp.blocks[j].num_peers > 1) color = esc("0;1");
+			else color = snubbed ? esc("0;35") : esc("0;33");
 
 #ifndef TORRENT_WINDOWS
-				static char const* const progress[] = {
-					"\u2581", "\u2582", "\u2583", "\u2584",
-					"\u2585", "\u2586", "\u2587", "\u2588"
-				};
-				chr = progress[pp->blocks[j].bytes_progress * 8 / pp->blocks[j].block_size];
+			static char const* const progress[] = {
+				"\u2581", "\u2582", "\u2583", "\u2584",
+				"\u2585", "\u2586", "\u2587", "\u2588"
+			};
+			chr = progress[pp.blocks[j].bytes_progress * 8 / pp.blocks[j].block_size];
 #else
-				static char const* const progress[] = { "\xb0", "\xb1", "\xb2" };
-				chr = progress[pp->blocks[j].bytes_progress * 3 / pp->blocks[j].block_size];
+			static char const* const progress[] = { "\xb0", "\xb1", "\xb2" };
+			chr = progress[pp.blocks[j].bytes_progress * 3 / pp.blocks[j].block_size];
 #endif
-			}
-			else if (pp->blocks[j].state == block_info::finished) color = esc("32;7");
-			else if (pp->blocks[j].state == block_info::writing) color = esc("36;7");
-			else if (pp->blocks[j].state == block_info::requested)
-			{
-				color = snubbed ? esc("0;35") : esc("0");
-				chr = "=";
-			}
-			else { color = esc("0"); chr = " "; }
 		}
+		else if (pp.blocks[j].state == block_info::finished) color = esc("32;7");
+		else if (pp.blocks[j].state == block_info::writing) color = esc("36;7");
+		else if (pp.blocks[j].state == block_info::requested)
+		{
+			color = snubbed ? esc("0;35") : esc("0");
+			chr = "=";
+		}
+		else { color = esc("0"); chr = " "; }
+
 		if (last_color != color)
 		{
 			out += color;
@@ -1105,20 +1125,13 @@ bool is_resume_file(std::string const& s)
 	return true;
 }
 
-int main(int argc, char* argv[])
+void print_usage()
 {
-#ifndef _WIN32
-	// sets the terminal to single-character mode
-	// and resets when destructed
-	set_keypress s;
-#endif
-
-	if (argc == 1)
-	{
-		std::fprintf(stderr, R"(usage: client_test [OPTIONS] [TORRENT|MAGNETURL]
+	std::fprintf(stderr, R"(usage: client_test [OPTIONS] [TORRENT|MAGNETURL]
 OPTIONS:
 
 CLIENT OPTIONS
+  -h                    print this message
   -f <log file>         logs all events to the given file
   -s <path>             sets the save path for downloads. This also determines
                         the resume data save directory. Torrents from the resume
@@ -1136,7 +1149,8 @@ CLIENT OPTIONS
                         are present and check hashes on-demand)
   -e <loops>            exit client after the specified number of iterations
                         through the main loop
-  -O                    print session stats counters to the log)"
+  -O                    print session stats counters to the log
+  -1                    exit on first torrent completing (useful for benchmarks))"
 #ifdef TORRENT_UTP_LOG_ENABLE
 R"(
   -q                    Enable uTP transport-level verbose logging
@@ -1158,8 +1172,7 @@ BITTORRENT OPTIONS
 
 NETWORK OPTIONS
   -x <file>             loads an emule IP-filter file
-  -Y                    Rate limit local peers
-)"
+  -Y                    Rate limit local peers)"
 #if TORRENT_USE_I2P
 R"(  -i <i2p-host>         the hostname to an I2P SAM bridge to use
 )"
@@ -1184,6 +1197,21 @@ examples:
   --alert_mask=error,dht,dht_log,dht_operation
   --alert_mask=all
 )") ;
+}
+
+} // anonymous namespace
+
+int main(int argc, char* argv[])
+{
+#ifndef _WIN32
+	// sets the terminal to single-character mode
+	// and resets when destructed
+	set_keypress s_;
+#endif
+
+	if (argc == 1)
+	{
+		print_usage();
 		return 0;
 	}
 
@@ -1196,20 +1224,15 @@ examples:
 	lt::session_params params;
 
 #ifndef TORRENT_DISABLE_DHT
-	params.dht_settings.privacy_lookups = true;
 
 	std::vector<char> in;
 	if (load_file(".ses_state", in))
-	{
-		lt::error_code ec;
-		lt::bdecode_node e = lt::bdecode(in, ec);
-		if (!ec) params = read_session_params(e, session_handle::save_dht_state);
-	}
+		params = read_session_params(in, session_handle::save_dht_state);
+
+	params.settings.set_bool(settings_pack::dht_privacy_lookups, true);
 #endif
 
 	auto& settings = params.settings;
-	settings.set_int(settings_pack::cache_size, cache_size);
-	settings.set_int(settings_pack::choking_algorithm, settings_pack::rate_based_choker);
 
 	settings.set_str(settings_pack::user_agent, "client_test/" LIBTORRENT_VERSION);
 	settings.set_int(settings_pack::alert_mask
@@ -1252,14 +1275,11 @@ examples:
 		{
 			// print all libtorrent settings and exit
 			print_settings(settings_pack::string_type_base
-				, settings_pack::num_string_settings
-				, "%s=<string>\n");
+				, settings_pack::num_string_settings, "string");
 			print_settings(settings_pack::bool_type_base
-				, settings_pack::num_bool_settings
-				, "%s=<bool>\n");
+				, settings_pack::num_bool_settings, "bool");
 			print_settings(settings_pack::int_type_base
-				, settings_pack::num_int_settings
-				, "%s=<int>\n");
+				, settings_pack::num_int_settings, "int");
 			return 0;
 		}
 
@@ -1269,34 +1289,47 @@ examples:
 			char const* equal = strchr(argv[i], '=');
 			char const* start = argv[i]+2;
 			// +2 is to skip the --
-			std::string const key(start, equal - start);
+			std::string const key(start, std::size_t(equal - start));
 			char const* value = equal + 1;
 
 			assign_setting(settings, key, value);
 			continue;
 		}
 
+		// command line switches that don't take an argument
+		switch (argv[i][1])
+		{
+			case 'k': settings = lt::high_performance_seed(); continue;
+			case 'G': seed_mode = true; continue;
+			case 'O': stats_enabled = true; continue;
+			case '1': exit_on_finish = true; continue;
+#ifdef TORRENT_UTP_LOG_ENABLE
+			case 'q':
+				lt::set_utp_stream_logging(true);
+				continue;
+#endif
+			case 'Q': share_mode = true; continue;
+			case 'Y': rate_limit_locals = true; continue;
+			case '0': params.disk_io_constructor = lt::disabled_disk_io_constructor; continue;
+			case 'h': print_usage(); return 0;
+		}
+
 		// if there's a flag but no argument following, ignore it
-		if (argc == i) continue;
+		if (argc == i + 1)
+		{
+			std::fprintf(stderr, "invalid command line argument or missing parameter: %s\n", argv[i]);
+			return 1;
+		}
 		char const* arg = argv[i+1];
 		if (arg == nullptr) arg = "";
 
 		switch (argv[i][1])
 		{
 			case 'f': g_log_file = std::fopen(arg, "w+"); break;
-			case 'k': settings = lt::high_performance_seed(); --i; break;
-			case 'G': seed_mode = true; --i; break;
 			case 's': save_path = make_absolute_path(arg); break;
-			case 'O': stats_enabled = true; --i; break;
-#ifdef TORRENT_UTP_LOG_ENABLE
-			case 'q':
-				lt::set_utp_stream_logging(true);
-				break;
-#endif
 			case 'U': torrent_upload_limit = atoi(arg) * 1000; break;
 			case 'D': torrent_download_limit = atoi(arg) * 1000; break;
 			case 'm': monitor_dir = make_absolute_path(arg); break;
-			case 'Q': share_mode = true; --i; break;
 			case 't': poll_interval = atoi(arg); break;
 			case 'F': refresh_delay = lt::milliseconds(atoi(arg)); break;
 			case 'a': allocation_mode = (arg == std::string("sparse"))
@@ -1308,7 +1341,7 @@ examples:
 					std::fstream filter(arg, std::ios_base::in);
 					if (!filter.fail())
 					{
-						std::regex regex(R"(^\s*([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)\s*-\s*([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)\s+([0-9]+)$)");
+						std::regex regex(R"(^\s*([0-9\.]+)\s*-\s*([0-9\.]+)\s+([0-9]+)$)");
 
 						std::string line;
 						while (std::getline(filter, line))
@@ -1316,10 +1349,9 @@ examples:
 							std::smatch m;
 							if (std::regex_match(line, m, regex))
 							{
-								address_v4 start((stoi(m[1]) << 24) | (stoi(m[2]) << 16) | (stoi(m[3]) << 8) | stoi(m[4]));
-								address_v4 last((stoi(m[5]) << 24) | (stoi(m[6]) << 16) | (stoi(m[7]) << 8) | stoi(m[8]));
-								loaded_ip_filter.add_rule(start, last
-									, stoi(m[9]) <= 127 ? lt::ip_filter::blocked : 0);
+								address_v4 start = make_address_v4(m[1]);
+								address_v4 last = make_address_v4(m[2]);
+								loaded_ip_filter.add_rule(start, last, stoi(m[3]) <= 127 ? lt::ip_filter::blocked : 0);
 							}
 						}
 					}
@@ -1327,13 +1359,6 @@ examples:
 				break;
 			case 'T': max_connections_per_torrent = atoi(arg); break;
 			case 'r': peer = arg; break;
-			case 'Y':
-				{
-					--i;
-					rate_limit_locals = true;
-					break;
-				}
-			case '0': disable_storage = true; --i; break;
 			case 'e':
 				{
 					loop_limit = atoi(arg);
@@ -1360,11 +1385,11 @@ examples:
 	if (rate_limit_locals)
 	{
 		lt::ip_filter pcf;
-		pcf.add_rule(address_v4::from_string("0.0.0.0")
-			, address_v4::from_string("255.255.255.255")
+		pcf.add_rule(make_address_v4("0.0.0.0")
+			, make_address_v4("255.255.255.255")
 			, 1 << static_cast<std::uint32_t>(lt::session::global_peer_class_id));
-		pcf.add_rule(address_v6::from_string("::")
-			, address_v6::from_string("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"), 1);
+		pcf.add_rule(make_address_v6("::")
+			, make_address_v6("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff"), 1);
 		ses.set_peer_class_filter(pcf);
 	}
 
@@ -1418,7 +1443,6 @@ examples:
 
 	// main loop
 	std::vector<lt::peer_info> peers;
-	std::vector<lt::partial_piece_info> queue;
 
 #ifndef _WIN32
 	signal(SIGTERM, signal_handler);
@@ -1547,7 +1571,7 @@ examples:
 
 #ifndef _WIN32
 					// enable terminal echo temporarily
-					set_keypress s(set_keypress::echo | set_keypress::canonical);
+					set_keypress echo_(set_keypress::echo | set_keypress::canonical);
 #endif
 					if (std::scanf("%4095s", url) == 1) add_magnet(ses, url);
 					else std::printf("failed to read magnet link\n");
@@ -1562,18 +1586,12 @@ examples:
 				if (c == 'W' && h.is_valid())
 				{
 					std::set<std::string> seeds = h.url_seeds();
-					for (std::set<std::string>::iterator i = seeds.begin()
-						, end(seeds.end()); i != end; ++i)
-					{
-						h.remove_url_seed(*i);
-					}
+					for (auto const& s : seeds)
+						h.remove_url_seed(s);
 
 					seeds = h.http_seeds();
-					for (std::set<std::string>::iterator i = seeds.begin()
-						, end(seeds.end()); i != end; ++i)
-					{
-						h.remove_http_seed(*i);
-					}
+					for (auto const& s : seeds)
+						h.remove_http_seed(s);
 				}
 
 				if (c == 'D' && h.is_valid())
@@ -1583,14 +1601,14 @@ examples:
 						, st.name.c_str());
 #ifndef _WIN32
 					// enable terminal echo temporarily
-					set_keypress s(set_keypress::echo | set_keypress::canonical);
+					set_keypress echo_(set_keypress::echo | set_keypress::canonical);
 #endif
 					char response = 'n';
 					int scan_ret = std::scanf("%c", &response);
 					if (scan_ret == 1 && response == 'y')
 					{
 						// also delete the resume file
-						std::string const rpath = resume_file(st.info_hash);
+						std::string const rpath = resume_file(st.info_hashes);
 						if (::remove(rpath.c_str()) < 0)
 							std::printf("failed to delete resume file (\"%s\")\n"
 								, rpath.c_str());
@@ -1705,13 +1723,6 @@ examples:
 				if (c == '6') print_fails = !print_fails;
 				if (c == '7') print_send_bufs = !print_send_bufs;
 				if (c == '8') print_local_ip = !print_local_ip;
-				if (c == 'C')
-				{
-					cache_size = (cache_size == 0) ? -1 : 0;
-					settings_pack p;
-					p.set_int(settings_pack::cache_size, cache_size);
-					ses.apply_settings(std::move(p));
-				}
 				if (c == 'h')
 				{
 					clear_screen();
@@ -1724,7 +1735,7 @@ CLIENT OPTIONS
 [q] quit client                                 [m] add magnet link
 
 TORRENT ACTIONS
-[p] pause/resume selected torrent               [C] toggle disk cache
+[p] pause/resume selected torrent               [W] remove all web seeds
 [s] toggle sequential download                  [j] force recheck
 [space] toggle session pause                    [c] clear error
 [v] scrape                                      [D] delete torrent and data
@@ -1770,12 +1781,8 @@ COLUMN OPTIONS
 		int pos = view.height() + ses_view.height();
 		set_cursor_pos(0, pos);
 
-		int cache_flags = print_downloads ? 0 : lt::session::disk_cache_no_pieces;
 		torrent_handle h = view.get_active_handle();
 
-		cache_status cs;
-		ses.get_cache_info(&cs, h, cache_flags);
-
 #ifndef TORRENT_DISABLE_DHT
 		if (show_dht_status)
 		{
@@ -1888,22 +1895,29 @@ COLUMN OPTIONS
 					for (auto const& ep : ae.endpoints)
 					{
 						++idx;
-						if (!ep.enabled) continue;
 						if (pos + 1 >= terminal_height) break;
+						if (!ep.enabled) continue;
+						for (lt::protocol_version const v : {lt::protocol_version::V1, lt::protocol_version::V2})
+						{
+							if (!s.info_hashes.has(v)) continue;
+							auto const& av = ep.info_hashes[v];
 
-						std::snprintf(str, sizeof(str), "  [%2d] fails: %-3d (%-3d) %s %5d \"%s\" %s\x1b[K\n"
-							, idx
-							, ep.fails, ae.fail_limit
-							, to_string(int(total_seconds(ep.next_announce - now)), 8).c_str()
-							, ep.min_announce > now ? int(total_seconds(ep.min_announce - now)) : 0
-							, ep.last_error ? ep.last_error.message().c_str() : ""
-							, ep.message.c_str());
-						out += str;
-						pos += 1;
-						// we only need to show this error once, not for every
-						// endpoint
-						if (ep.last_error == boost::asio::error::host_not_found) break;
+							std::snprintf(str, sizeof(str), "  [%2d] fails: %-3d (%-3d) %s %5d \"%s\" %s\x1b[K\n"
+								, idx
+								, av.fails, ae.fail_limit
+								, to_string(int(total_seconds(av.next_announce - now)), 8).c_str()
+								, av.min_announce > now ? int(total_seconds(av.min_announce - now)) : 0
+								, av.last_error ? av.last_error.message().c_str() : ""
+								, av.message.c_str());
+							out += str;
+							pos += 1;
+							// we only need to show this error once, not for every
+							// endpoint
+							if (av.last_error == boost::asio::error::host_not_found)
+								goto done;
+						}
 					}
+done:
 
 					if (pos + 1 >= terminal_height) break;
 				}
@@ -1918,61 +1932,16 @@ COLUMN OPTIONS
 
 			if (print_downloads)
 			{
-				h.get_download_queue(queue);
-
-				std::sort(queue.begin(), queue.end()
-					, [] (lt::partial_piece_info const& lhs, lt::partial_piece_info const& rhs)
-					{ return lhs.piece_index < rhs.piece_index; });
-
-				std::sort(cs.pieces.begin(), cs.pieces.end()
-					, [](lt::cached_piece_info const& lhs, lt::cached_piece_info const& rhs)
-					{ return lhs.piece < rhs.piece; });
+				std::vector<lt::partial_piece_info> queue = h.get_download_queue();
 
 				int p = 0; // this is horizontal position
-				for (lt::cached_piece_info const& i : cs.pieces)
-				{
-					if (pos + 3 >= terminal_height) break;
-
-					lt::partial_piece_info* pp = nullptr;
-					lt::partial_piece_info tmp;
-					tmp.piece_index = i.piece;
-					std::vector<lt::partial_piece_info>::iterator ppi
-						= std::lower_bound(queue.begin(), queue.end(), tmp
-						, [](lt::partial_piece_info const& lhs, lt::partial_piece_info const& rhs)
-						{ return lhs.piece_index < rhs.piece_index; });
-
-					if (ppi != queue.end() && ppi->piece_index == i.piece) pp = &*ppi;
-
-					print_piece(pp, &i, peers, out);
-
-					int num_blocks = pp ? pp->blocks_in_piece : int(i.blocks.size());
-					p += num_blocks + 8;
-					bool continuous_mode = 8 + num_blocks > terminal_width;
-					if (continuous_mode)
-					{
-						while (p > terminal_width)
-						{
-							p -= terminal_width;
-							++pos;
-						}
-					}
-					else if (p + num_blocks + 8 > terminal_width)
-					{
-						out += "\x1b[K\n";
-						pos += 1;
-						p = 0;
-					}
-
-					if (pp) queue.erase(ppi);
-				}
-
 				for (lt::partial_piece_info const& i : queue)
 				{
 					if (pos + 3 >= terminal_height) break;
 
-					print_piece(&i, nullptr, peers, out);
+					print_piece(i, peers, out);
 
-					int num_blocks = i.blocks_in_piece;
+					int const num_blocks = i.blocks_in_piece;
 					p += num_blocks + 8;
 					bool continuous_mode = 8 + num_blocks > terminal_width;
 					if (continuous_mode)
@@ -1996,10 +1965,9 @@ COLUMN OPTIONS
 					pos += 1;
 				}
 
-				std::snprintf(str, sizeof(str), "%s %s read cache | %s %s downloading | %s %s cached | %s %s flushed | %s %s snubbed | = requested\x1b[K\n"
-					, esc("34;7"), esc("0") // read cache
+				std::snprintf(str, sizeof(str), "%s %s downloading | %s %s writing | %s %s flushed | %s %s snubbed | = requested\x1b[K\n"
 					, esc("33;7"), esc("0") // downloading
-					, esc("36;7"), esc("0") // cached
+					, esc("36;7"), esc("0") // writing
 					, esc("32;7"), esc("0") // flushed
 					, esc("35;7"), esc("0") // snubbed
 					);
@@ -2009,8 +1977,7 @@ COLUMN OPTIONS
 
 			if (print_file_progress && s.has_metadata)
 			{
-				std::vector<std::int64_t> file_progress;
-				h.file_progress(file_progress);
+				std::vector<std::int64_t> const file_progress = h.file_progress();
 				std::vector<lt::open_file_state> file_status = h.file_status();
 				std::vector<lt::download_priority_t> file_prio = h.get_file_priorities();
 				auto f = file_status.begin();
@@ -2019,7 +1986,7 @@ COLUMN OPTIONS
 				int p = 0; // this is horizontal position
 				for (file_index_t i(0); i < file_index_t(ti->num_files()); ++i)
 				{
-					int const idx = static_cast<int>(i);
+					auto const idx = std::size_t(static_cast<int>(i));
 					if (pos + 1 >= terminal_height) break;
 
 					bool const pad_file = ti->files().pad_file_at(i);
@@ -2034,7 +2001,7 @@ COLUMN OPTIONS
 					std::string title = ti->files().file_name(i).to_string();
 					if (!complete)
 					{
-						std::snprintf(str, sizeof(str), " (%.1f%%)", progress / 10.f);
+						std::snprintf(str, sizeof(str), " (%.1f%%)", progress / 10.0);
 						title += str;
 					}
 
@@ -2113,9 +2080,7 @@ COLUMN OPTIONS
 	std::vector<torrent_status> const temp = ses.get_torrent_status(
 		[](torrent_status const& st)
 		{
-			if (!st.handle.is_valid() || !st.has_metadata || !st.need_save_resume)
-				return false;
-			return true;
+			return st.handle.is_valid() && st.has_metadata && st.need_save_resume;
 		}, {});
 
 	int idx = 0;
@@ -2146,11 +2111,7 @@ COLUMN OPTIONS
 #ifndef TORRENT_DISABLE_DHT
 	std::printf("\nsaving session state\n");
 	{
-		lt::entry session_state;
-		ses.save_state(session_state, lt::session::save_dht_state);
-
-		std::vector<char> out;
-		bencode(std::back_inserter(out), session_state);
+		std::vector<char> out = write_session_params_buf(ses.session_state(lt::session::save_dht_state));
 		save_file(".ses_state", out);
 	}
 #endif
