@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2010-2020, Arvid Norberg
+Copyright (c) 2015, Mike Tzou
+Copyright (c) 2016-2017, Steven Siloti
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016, 2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,7 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/peer_id.hpp"
-#include "libtorrent/io_service.hpp"
+#include "libtorrent/io_context.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/address.hpp"
 #include "libtorrent/error_code.hpp"
@@ -40,8 +44,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/socket_io.hpp"
-#include "libtorrent/file_pool.hpp"
 #include "libtorrent/string_view.hpp"
+#include "libtorrent/session.hpp" // for default_disk_io_constructor
+#include "libtorrent/disk_interface.hpp"
+#include "libtorrent/performance_counters.hpp"
+#include "libtorrent/aux_/session_settings.hpp"
 #include <random>
 #include <cstring>
 #include <thread>
@@ -51,19 +58,23 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <array>
 #include <chrono>
 
-#if BOOST_ASIO_DYN_LINK
+#ifdef BOOST_ASIO_DYN_LINK
 #include <boost/asio/impl/src.hpp>
 #endif
 
+namespace {
+
 using namespace lt;
-using namespace lt::detail; // for write_* and read_*
+using namespace lt::aux; // for write_* and read_*
+using lt::make_address_v4;
 
 using namespace std::placeholders;
 
 void generate_block(span<std::uint32_t> buffer, piece_index_t const piece
 	, int const offset)
 {
-	std::uint32_t const fill = (static_cast<int>(piece) << 8) | ((offset / 0x4000) & 0xff);
+	std::uint32_t const fill = static_cast<std::uint32_t>(
+		(static_cast<int>(piece) << 8) | ((offset / 0x4000) & 0xff));
 	for (auto& w : buffer) w = fill;
 }
 
@@ -116,7 +127,7 @@ std::string leaf_path(std::string f)
 	{
 		// if the last character is a / (or \)
 		// ignore it
-		int len = 0;
+		std::size_t len = 0;
 		while (sep > first)
 		{
 			--sep;
@@ -140,7 +151,7 @@ std::mt19937 rng(dev());
 
 struct peer_conn
 {
-	peer_conn(io_service& ios, int num_pieces, int blocks_pp, tcp::endpoint const& ep
+	peer_conn(io_context& ios, int piece_count, int blocks_pp, tcp::endpoint const& ep
 		, char const* ih, bool seed_, int churn_, bool corrupt_)
 		: s(ios)
 		, read_pos(0)
@@ -156,7 +167,7 @@ struct peer_conn
 		, fast_extension(false)
 		, blocks_received(0)
 		, blocks_sent(0)
-		, num_pieces(num_pieces)
+		, num_pieces(piece_count)
 		, start_time(clock_type::now())
 		, churn(churn_)
 		, corrupt(corrupt_)
@@ -165,7 +176,7 @@ struct peer_conn
 	{
 		corruption_counter = rand() % 1000;
 		if (seed) ++num_seeds;
-		pieces.reserve(num_pieces);
+		pieces.reserve(std::size_t(piece_count));
 		start_conn();
 	}
 
@@ -177,18 +188,16 @@ struct peer_conn
 			s.open(endpoint.protocol(), ec);
 			if (ec)
 			{
-				close("ERROR OPEN: %s", ec);
+				close("ERROR OPEN", ec);
 				return;
 			}
 			tcp::endpoint bind_if(address_v4(
-				(127 << 24)
-				+ ((local_if_counter / 255) << 16)
-				+ ((local_if_counter % 255) + 1)), 0);
+				(127 << 24) + unsigned (local_if_counter + 1)), 0);
 			++local_if_counter;
 			s.bind(bind_if, ec);
 			if (ec)
 			{
-				close("ERROR BIND: %s", ec);
+				close("ERROR BIND", ec);
 				return;
 			}
 		}
@@ -237,7 +246,7 @@ struct peer_conn
 	{
 		if (ec)
 		{
-			close("ERROR CONNECT: %s", ec);
+			close("ERROR CONNECT", ec);
 			return;
 		}
 
@@ -245,10 +254,10 @@ struct peer_conn
 			"                    " // space for info-hash
 			"aaaaaaaaaaaaaaaaaaaa" // peer-id
 			"\0\0\0\x01\x02"; // interested
-		char* h = (char*)malloc(sizeof(handshake));
+		char* h = static_cast<char*>(malloc(sizeof(handshake)));
 		memcpy(h, handshake, sizeof(handshake));
 		std::memcpy(h + 28, info_hash, 20);
-		std::generate(h + 48, h + 68, &rand);
+		std::generate(h + 48, h + 68, [] { return char(rand()); });
 		// for seeds, don't send the interested message
 		boost::asio::async_write(s, boost::asio::buffer(h, (sizeof(handshake) - 1) - (seed ? 5 : 0))
 			, std::bind(&peer_conn::on_handshake, this, h, _1, _2));
@@ -259,12 +268,12 @@ struct peer_conn
 		free(h);
 		if (ec)
 		{
-			close("ERROR SEND HANDSHAKE: %s", ec);
+			close("ERROR SEND HANDSHAKE", ec);
 			return;
 		}
 
 		// read handshake
-		boost::asio::async_read(s, boost::asio::buffer((char*)buffer, 68)
+		boost::asio::async_read(s, boost::asio::buffer(buffer, 68)
 			, std::bind(&peer_conn::on_handshake2, this, _1, _2));
 	}
 
@@ -272,14 +281,14 @@ struct peer_conn
 	{
 		if (ec)
 		{
-			close("ERROR READ HANDSHAKE: %s", ec);
+			close("ERROR READ HANDSHAKE", ec);
 			return;
 		}
 
 		// buffer is the full 68 byte handshake
 		// look at the extension bits
 
-		fast_extension = (((char*)buffer)[27] & 4) != 0;
+		fast_extension = (reinterpret_cast<char const*>(buffer)[27] & 4) != 0;
 
 		if (seed)
 		{
@@ -302,22 +311,22 @@ struct peer_conn
 			// unchoke
 			write_uint32(1, ptr);
 			write_uint8(1, ptr);
-			boost::asio::async_write(s, boost::asio::buffer(write_buf_proto, ptr - write_buf_proto)
+			boost::asio::async_write(s, boost::asio::buffer(write_buf_proto, std::size_t(ptr - write_buf_proto))
 				, std::bind(&peer_conn::on_have_all_sent, this, _1, _2));
 		}
 		else
 		{
 			// bitfield
 			int len = (num_pieces + 7) / 8;
-			char* ptr = (char*)buffer;
+			char* ptr = reinterpret_cast<char*>(buffer);
 			write_uint32(len + 1, ptr);
 			write_uint8(5, ptr);
-			memset(ptr, 255, len);
+			memset(ptr, 255, std::size_t(len));
 			ptr += len;
 			// unchoke
 			write_uint32(1, ptr);
 			write_uint8(1, ptr);
-			boost::asio::async_write(s, boost::asio::buffer((char*)buffer, len + 10)
+			boost::asio::async_write(s, boost::asio::buffer(buffer, std::size_t(len + 10))
 				, std::bind(&peer_conn::on_have_all_sent, this, _1, _2));
 		}
 	}
@@ -326,12 +335,12 @@ struct peer_conn
 	{
 		if (ec)
 		{
-			close("ERROR SEND HAVE ALL: %s", ec);
+			close("ERROR SEND HAVE ALL", ec);
 			return;
 		}
 
 		// read message
-		boost::asio::async_read(s, boost::asio::buffer((char*)buffer, 4)
+		boost::asio::async_read(s, boost::asio::buffer(buffer, 4)
 			, std::bind(&peer_conn::on_msg_length, this, _1, _2));
 	}
 
@@ -378,7 +387,7 @@ struct peer_conn
 			"    " // piece
 			"    " // offset
 			"    "; // length
-		char* m = (char*)malloc(sizeof(msg));
+		char* m = static_cast<char*>(malloc(sizeof(msg)));
 		memcpy(m, msg, sizeof(msg));
 		char* ptr = m + 5;
 		write_uint32(static_cast<int>(current_piece), ptr);
@@ -403,31 +412,31 @@ struct peer_conn
 		free(m);
 		if (ec)
 		{
-			close("ERROR SEND REQUEST: %s", ec);
+			close("ERROR SEND REQUEST", ec);
 			return;
 		}
 
 		work_download();
 	}
 
-	void close(char const* fmt, error_code const& ec)
+	void close(char const* msg, error_code const& ec)
 	{
 		end_time = clock_type::now();
 		char tmp[1024];
-		std::snprintf(tmp, sizeof(tmp), fmt, ec.message().c_str());
+		std::snprintf(tmp, sizeof(tmp), "%s: %s", msg, ec ? ec.message().c_str() : "");
 		int time = int(total_milliseconds(end_time - start_time));
 		if (time == 0) time = 1;
-		float up = (std::int64_t(blocks_sent) * 0x4000) / time / 1000.f;
-		float down = (std::int64_t(blocks_received) * 0x4000) / time / 1000.f;
+		double const up = (std::int64_t(blocks_sent) * 0x4000) / time / 1000.0;
+		double const down = (std::int64_t(blocks_received) * 0x4000) / time / 1000.0;
 		error_code e;
 
 		char ep_str[200];
 		address const& addr = s.local_endpoint(e).address();
 		if (addr.is_v6())
-			std::snprintf(ep_str, sizeof(ep_str), "[%s]:%d", addr.to_string(e).c_str()
+			std::snprintf(ep_str, sizeof(ep_str), "[%s]:%d", addr.to_string().c_str()
 				, s.local_endpoint(e).port());
 		else
-			std::snprintf(ep_str, sizeof(ep_str), "%s:%d", addr.to_string(e).c_str()
+			std::snprintf(ep_str, sizeof(ep_str), "%s:%d", addr.to_string().c_str()
 				, s.local_endpoint(e).port());
 		std::printf("%s ep: %s sent: %d received: %d duration: %d ms up: %.1fMB/s down: %.1fMB/s\n"
 			, tmp, ep_str, blocks_sent, blocks_received, time, up, down);
@@ -453,7 +462,7 @@ struct peer_conn
 		}
 
 		// read message
-		boost::asio::async_read(s, boost::asio::buffer((char*)buffer, 4)
+		boost::asio::async_read(s, boost::asio::buffer(buffer, 4)
 			, std::bind(&peer_conn::on_msg_length, this, _1, _2));
 	}
 
@@ -468,10 +477,10 @@ struct peer_conn
 
 		if (ec)
 		{
-			close("ERROR RECEIVE MESSAGE PREFIX: %s", ec);
+			close("ERROR RECEIVE MESSAGE PREFIX", ec);
 			return;
 		}
-		char* ptr = (char*)buffer;
+		char* ptr = reinterpret_cast<char*>(buffer);
 		unsigned int length = read_uint32(ptr);
 		if (length > sizeof(buffer))
 		{
@@ -479,7 +488,7 @@ struct peer_conn
 			close("ERROR RECEIVE MESSAGE PREFIX: packet too big", error_code());
 			return;
 		}
-		boost::asio::async_read(s, boost::asio::buffer((char*)buffer, length)
+		boost::asio::async_read(s, boost::asio::buffer(buffer, length)
 			, std::bind(&peer_conn::on_message, this, _1, _2));
 	}
 
@@ -494,10 +503,10 @@ struct peer_conn
 
 		if (ec)
 		{
-			close("ERROR RECEIVE MESSAGE: %s", ec);
+			close("ERROR RECEIVE MESSAGE", ec);
 			return;
 		}
-		char* ptr = (char*)buffer;
+		char* ptr = reinterpret_cast<char*>(buffer);
 		int msg = read_uint8(ptr);
 
 		if (test_mode == dual_test && num_seeds == 0)
@@ -518,9 +527,9 @@ struct peer_conn
 					close("REQUEST packet has invalid size", error_code());
 					return;
 				}
-				piece_index_t const piece = piece_index_t(detail::read_int32(ptr));
-				int const start = detail::read_int32(ptr);
-				int const length = detail::read_int32(ptr);
+				piece_index_t const piece = piece_index_t(aux::read_int32(ptr));
+				int const start = aux::read_int32(ptr);
+				int const length = aux::read_int32(ptr);
 				write_piece(piece, start, length);
 			}
 			else if (msg == 3) // not-interested
@@ -540,20 +549,20 @@ struct peer_conn
 			if (msg == 0xe) // have_all
 			{
 				// build a list of all pieces and request them all!
-				pieces.resize(num_pieces);
-				for (piece_index_t i(0); i < piece_index_t(int(pieces.size())); ++i)
-					pieces[static_cast<int>(i)] = i;
+				pieces.resize(std::size_t(num_pieces));
+				for (std::size_t i = 0; i < pieces.size(); ++i)
+					pieces[i] = piece_index_t(int(i));
 				std::shuffle(pieces.begin(), pieces.end(), rng);
 			}
 			else if (msg == 4) // have
 			{
-				piece_index_t const piece(detail::read_int32(ptr));
+				piece_index_t const piece(aux::read_int32(ptr));
 				if (pieces.empty()) pieces.push_back(piece);
-				else pieces.insert(pieces.begin() + (rand() % pieces.size()), piece);
+				else pieces.insert(pieces.begin() + (unsigned(rand()) % pieces.size()), piece);
 			}
 			else if (msg == 5) // bitfield
 			{
-				pieces.reserve(num_pieces);
+				pieces.reserve(std::size_t(num_pieces));
 				piece_index_t piece(0);
 				for (int i = 0; i < int(bytes_transferred); ++i)
 				{
@@ -573,15 +582,15 @@ struct peer_conn
 			{
 				if (verify_downloads)
 				{
-					piece_index_t const piece(read_uint32(ptr));
-					int start = read_uint32(ptr);
+					piece_index_t const piece(read_int32(ptr));
+					int start = read_int32(ptr);
 					int size = int(bytes_transferred) - 9;
 					verify_piece(piece, start, ptr, size);
 				}
 				++blocks_received;
 				--outstanding_requests;
-				piece_index_t const piece = piece_index_t(detail::read_int32(ptr));
-				int start = detail::read_int32(ptr);
+				piece_index_t const piece = piece_index_t(aux::read_int32(ptr));
+				int start = aux::read_int32(ptr);
 
 				if (churn && (blocks_received % churn) == 0) {
 					outstanding_requests = 0;
@@ -589,7 +598,7 @@ struct peer_conn
 					s.close();
 					return;
 				}
-				if (int((start + bytes_transferred) / 0x4000) == blocks_per_piece)
+				if ((start + int(bytes_transferred)) / 0x4000 == blocks_per_piece)
 				{
 					write_have(piece);
 					return;
@@ -597,7 +606,7 @@ struct peer_conn
 			}
 			else if (msg == 13) // suggest
 			{
-				piece_index_t const piece(detail::read_int32(ptr));
+				piece_index_t const piece(aux::read_int32(ptr));
 				auto i = std::find(pieces.begin(), pieces.end(), piece);
 				if (i != pieces.end())
 				{
@@ -608,9 +617,9 @@ struct peer_conn
 			}
 			else if (msg == 16) // reject request
 			{
-				piece_index_t const piece(detail::read_int32(ptr));
-				int start = detail::read_int32(ptr);
-				int length = detail::read_int32(ptr);
+				piece_index_t const piece(aux::read_int32(ptr));
+				int start = aux::read_int32(ptr);
+				int length = aux::read_int32(ptr);
 
 				// put it back!
 				if (current_piece != piece)
@@ -642,7 +651,7 @@ struct peer_conn
 			}
 			else if (msg == 17) // allowed_fast
 			{
-				piece_index_t const piece = piece_index_t(detail::read_int32(ptr));
+				piece_index_t const piece = piece_index_t(aux::read_int32(ptr));
 				auto i = std::find(pieces.begin(), pieces.end(), piece);
 				if (i != pieces.end())
 				{
@@ -656,8 +665,9 @@ struct peer_conn
 
 	bool verify_piece(piece_index_t const piece, int start, char const* ptr, int size)
 	{
-		std::uint32_t* buf = (std::uint32_t*)ptr;
-		std::uint32_t const fill = (static_cast<int>(piece) << 8) | ((start / 0x4000) & 0xff);
+		std::uint32_t const* buf = reinterpret_cast<std::uint32_t const*>(ptr);
+		std::uint32_t const fill = static_cast<std::uint32_t>(
+			(static_cast<int>(piece) << 8) | ((start / 0x4000) & 0xff));
 		for (int i = 0; i < size / 4; ++i)
 		{
 			if (buf[i] != fill)
@@ -691,8 +701,8 @@ struct peer_conn
 		write_uint32(static_cast<int>(piece), ptr);
 		write_uint32(start, ptr);
 		std::array<boost::asio::const_buffer, 2> vec;
-		vec[0] = boost::asio::buffer(write_buf_proto, ptr - write_buf_proto);
-		vec[1] = boost::asio::buffer(write_buffer, length);
+		vec[0] = boost::asio::buffer(write_buf_proto, std::size_t(ptr - write_buf_proto));
+		vec[1] = boost::asio::buffer(write_buffer, std::size_t(length));
 		boost::asio::async_write(s, vec, std::bind(&peer_conn::on_have_all_sent, this, _1, _2));
 		++blocks_sent;
 		if (churn && (blocks_sent % churn) == 0 && seed) {
@@ -712,7 +722,7 @@ struct peer_conn
 	}
 };
 
-void print_usage()
+[[noreturn]] void print_usage()
 {
 	std::fprintf(stderr, "usage: connection_tester command [options]\n\n"
 		"command is one of:\n"
@@ -720,11 +730,7 @@ void print_usage()
 		"    options for this command:\n"
 		"    -s <size>          the size of the torrent in megabytes\n"
 		"    -n <num-files>     the number of files in the test torrent\n"
-		"    -a                 introduce a lot of pad-files\n"
-		"                       (pad files are not supported for gen-data or upload)\n"
 		"    -t <file>          the file to save the .torrent file to\n"
-		"    -T <name>          the name of the torrent (and directory\n"
-		"                       its files are saved in)\n\n"
 		"  gen-data             generate the data file(s) for the test torrent\n"
 		"    options for this command:\n"
 		"    -t <file>          the torrent file that was previously generated\n"
@@ -754,25 +760,61 @@ void print_usage()
 	exit(1);
 }
 
-void hasher_thread(lt::create_torrent* t, piece_index_t const start_piece
-	, piece_index_t const end_piece, int piece_size, bool print)
+void hasher_thread(lt::aux::vector<sha1_hash, piece_index_t>* output
+	, lt::file_storage const& fs
+	, piece_index_t const start_piece
+	, piece_index_t const end_piece
+	, bool print)
 {
 	if (print) std::fprintf(stderr, "\n");
 	std::uint32_t piece[0x4000 / 4];
+	int const piece_size = fs.piece_length();
+
+	std::vector<file_slice> files = fs.map_block(start_piece, 0
+		, std::min(static_cast<int>(end_piece - start_piece) * std::int64_t(piece_size)
+			, fs.total_size() - static_cast<int>(start_piece) * std::int64_t(piece_size)));
+
 	for (piece_index_t i = start_piece; i < end_piece; ++i)
 	{
 		hasher ph;
 		for (int j = 0; j < piece_size; j += 0x4000)
 		{
 			generate_block(piece, i, j);
+
+			// if any part of this block overlaps with a pad-file, we need to
+			// clear those bytes to 0
+			for (int k = 0; k < 0x4000; )
+			{
+				if (files.empty())
+				{
+					TORRENT_ASSERT(i == prev(end_piece));
+					TORRENT_ASSERT(k > 0);
+					TORRENT_ASSERT(k < 0x4000);
+					// this is the last piece of the torrent, and the piece
+					// extends a bit past the end of the last file. This part
+					// should be truncated
+					ph.update(reinterpret_cast<char*>(piece), k);
+					goto out;
+				}
+				auto& f = files.front();
+				int const range = int(std::min(std::int64_t(0x4000 - k), f.size));
+				if (fs.pad_file_at(f.file_index))
+					std::memset(reinterpret_cast<char*>(piece) + k, 0, std::size_t(range));
+
+				f.offset += range;
+				f.size -= range;
+				k += range;
+				if (f.size == 0) files.erase(files.begin());
+			}
 			ph.update(reinterpret_cast<char*>(piece), 0x4000);
 		}
-		t->set_hash(i, ph.final());
+out:
+		(*output)[i] = ph.final();
 		int const range = static_cast<int>(end_piece) - static_cast<int>(start_piece);
 		if (print && (static_cast<int>(i) & 1))
 		{
 			int const delta_piece = static_cast<int>(i) - static_cast<int>(start_piece);
-			std::fprintf(stderr, "\r%.1f %% ", float(delta_piece * 100) / float(range));
+			std::fprintf(stderr, "\r%.1f %% ", double(delta_piece * 100) / double(range));
 		}
 	}
 	if (print) std::fprintf(stderr, "\n");
@@ -780,7 +822,7 @@ void hasher_thread(lt::create_torrent* t, piece_index_t const start_piece
 
 // size is in megabytes
 void generate_torrent(std::vector<char>& buf, int num_pieces, int num_files
-	, char const* torrent_name, bool with_padding)
+	, char const* torrent_name)
 {
 	file_storage fs;
 	// 1 MiB piece size
@@ -800,37 +842,88 @@ void generate_torrent(std::vector<char>& buf, int num_pieces, int num_files
 		file_size += 200;
 	}
 
-	lt::create_torrent t(fs, piece_size, with_padding ? 100 : -1);
+	lt::create_torrent t(fs, piece_size, lt::create_torrent::v1_only);
 
 	num_pieces = t.num_pieces();
 
 	int const num_threads = std::thread::hardware_concurrency()
-		? std::thread::hardware_concurrency() : 4;
+		? int(std::thread::hardware_concurrency()) : 4;
 	std::printf("hashing in %d threads\n", num_threads);
 
 	std::vector<std::thread> threads;
-	threads.reserve(num_threads);
+	threads.reserve(std::size_t(num_threads));
+	lt::aux::vector<lt::sha1_hash, piece_index_t> hashes{static_cast<std::size_t>(num_pieces)};
 	for (int i = 0; i < num_threads; ++i)
 	{
-		threads.emplace_back(&hasher_thread, &t
+		threads.emplace_back(&hasher_thread, &hashes, t.files()
 			, piece_index_t(i * num_pieces / num_threads)
 			, piece_index_t((i + 1) * num_pieces / num_threads)
-			, piece_size
 			, i == 0);
 	}
 
 	for (auto& i : threads)
 		i.join();
 
-	std::back_insert_iterator<std::vector<char>> out(buf);
-	bencode(out, t.generate());
+	for (auto i : t.files().piece_range())
+		t.set_hash(i, hashes[i]);
+
+	bencode(std::back_inserter(buf), t.generate());
+}
+
+void write_handler(file_storage const& fs
+	, disk_interface& disk, storage_holder& st
+	, piece_index_t& piece, int& offset
+	, lt::storage_error const& error)
+{
+	if (error)
+	{
+		std::fprintf(stderr, "storage error: %s\n", error.ec.message().c_str());
+		return;
+	}
+
+
+	if (static_cast<int>(piece) & 1)
+	{
+		std::fprintf(stderr, "\r%.1f %% "
+			, double(static_cast<int>(piece) * 100) / double(fs.num_pieces()));
+	}
+
+	if (piece >= fs.end_piece()) return;
+	offset += 0x4000;
+	if (offset >= fs.piece_size(piece))
+	{
+		offset = 0;
+		++piece;
+	}
+	if (piece >= fs.end_piece())
+	{
+		disk.abort(false);
+		return;
+	}
+
+	std::uint32_t buffer[0x4000 / 4];
+	generate_block(buffer, piece, offset);
+
+	int const left_in_piece = fs.piece_size(piece) - offset;
+	if (left_in_piece <= 0) return;
+
+	disk.async_write(st, { piece, offset, std::min(left_in_piece, 0x4000)}
+		, reinterpret_cast<char const*>(buffer)
+		, std::shared_ptr<disk_observer>()
+		, [&](lt::storage_error const& e)
+		{ write_handler(fs, disk, st, piece, offset, e); });
+
+	disk.submit_jobs();
 }
 
 void generate_data(char const* path, torrent_info const& ti)
 {
-	file_storage const& fs = ti.files();
+	io_context ios;
+	counters stats_counters;
+	settings_pack sett = default_settings();
+	std::unique_ptr<lt::disk_interface> disk = default_disk_io_constructor(ios, sett, stats_counters);
 
-	file_pool fp;
+	file_storage const& fs = ti.files();
 
 	aux::vector<download_priority_t, file_index_t> priorities;
 	sha1_hash info_hash;
@@ -843,40 +936,41 @@ void generate_data(char const* path, torrent_info const& ti)
 		info_hash
 	};
 
-	std::unique_ptr<storage_interface> st(default_storage_constructor(params, fp));
+	storage_holder st = disk->new_torrent(params, std::shared_ptr<void>());
 
+	piece_index_t piece(0);
+	int offset = 0;
+
+	std::uint32_t buffer[0x4000 / 4];
+	generate_block(buffer, piece, offset);
+
+	disk->async_write(st, { piece, offset, std::min(fs.piece_size(piece), 0x4000)}
+		, reinterpret_cast<char const*>(buffer)
+		, std::shared_ptr<disk_observer>()
+		, [&](lt::storage_error const& error)
+		{ write_handler(fs, *disk, st, piece, offset, error); });
+
+	// keep 10 writes in flight at all times
+	for (int i = 0; i < 10; ++i)
 	{
-		storage_error error;
-		st->initialize(error);
+		write_handler(fs, *disk, st, piece, offset, lt::storage_error());
 	}
 
-	std::uint32_t piece[0x4000 / 4];
-	for (piece_index_t i(0); i < piece_index_t(ti.num_pieces()); ++i)
-	{
-		for (int j = 0; j < ti.piece_size(i); j += 0x4000)
-		{
-			generate_block(piece, i, j);
-			int const left_in_piece = ti.piece_size(i) - j;
-			iovec_t const b = { reinterpret_cast<char*>(piece)
-				, std::min(left_in_piece, 0x4000)};
-			storage_error error;
-			st->writev(b, i, j, open_mode::write_only, error);
-			if (error)
-				std::fprintf(stderr, "storage error: %s\n", error.ec.message().c_str());
-		}
-		if (static_cast<int>(i) & 1)
-		{
-			std::fprintf(stderr, "\r%.1f %% ", float(static_cast<int>(i) * 100) / float(ti.num_pieces()));
-		}
-	}
+	disk->submit_jobs();
+
+	ios.run();
 }
 
-void io_thread(io_service* ios)
+void io_thread(io_context* ios) try
 {
-	error_code ec;
-	ios->run(ec);
-	if (ec) std::fprintf(stderr, "ERROR: %s\n", ec.message().c_str());
+	ios->run();
 }
+catch (std::exception const& e)
+{
+	std::fprintf(stderr, "ERROR: %s\n", e.what());
+}
+
+} // anonymous namespace
 
 int main(int argc, char* argv[])
 {
@@ -892,7 +986,6 @@ int main(int argc, char* argv[])
 	char const* destination_ip = "127.0.0.1";
 	int destination_port = 6881;
 	int churn = 0;
-	bool gen_pad_files = false;
 	std::vector<std::string> trackers;
 
 	argv += 2;
@@ -914,7 +1007,6 @@ int main(int argc, char* argv[])
 		switch (optname[1])
 		{
 			case 'C': test_corruption = true; continue;
-			case 'a': gen_pad_files = true; continue;
 		}
 
 		if (argc == 0)
@@ -923,22 +1015,22 @@ int main(int argc, char* argv[])
 			break;
 		}
 
-		char const* optarg = argv[0];
+		char const* opt = argv[0];
 		++argv;
 		--argc;
 
 		switch (optname[1])
 		{
-			case 's': size = atoi(optarg); break;
-			case 'n': num_files = atoi(optarg); break;
-			case 'N': num_torrents = atoi(optarg); break;
-			case 't': torrent_file = optarg; break;
-			case 'T': trackers.push_back(optarg); break;
-			case 'P': data_path = optarg; break;
-			case 'c': num_connections = atoi(optarg); break;
-			case 'p': destination_port = atoi(optarg); break;
-			case 'd': destination_ip = optarg; break;
-			case 'r': churn = atoi(optarg); break;
+			case 's': size = atoi(opt); break;
+			case 'n': num_files = atoi(opt); break;
+			case 'N': num_torrents = atoi(opt); break;
+			case 't': torrent_file = opt; break;
+			case 'T': trackers.push_back(opt); break;
+			case 'P': data_path = opt; break;
+			case 'c': num_connections = atoi(opt); break;
+			case 'p': destination_port = atoi(opt); break;
+			case 'd': destination_ip = opt; break;
+			case 'r': churn = atoi(opt); break;
 			default: std::fprintf(stderr, "unknown option: %s\n", optname);
 		}
 	}
@@ -950,7 +1042,7 @@ int main(int argc, char* argv[])
 		name = name.substr(0, name.find_last_of('.'));
 		std::printf("generating torrent: %s\n", name.c_str());
 		generate_torrent(tmp, size ? size : 1024, num_files ? num_files : 1
-			, name.c_str(), gen_pad_files);
+			, name.c_str() );
 
 		FILE* output = stdout;
 		if ("-"_sv != torrent_file)
@@ -998,7 +1090,7 @@ int main(int argc, char* argv[])
 			}
 			// 1 MiB piece size
 			const int piece_size = 1024 * 1024;
-			lt::create_torrent t(fs, piece_size);
+			lt::create_torrent t(fs, piece_size, lt::create_torrent::v1_only);
 			sha1_hash zero(nullptr);
 			for (auto const k : fs.piece_range())
 				t.set_hash(k, zero);
@@ -1048,7 +1140,7 @@ int main(int argc, char* argv[])
 	}
 
 	error_code ec;
-	address_v4 addr = address_v4::from_string(destination_ip, ec);
+	address_v4 addr = make_address_v4(destination_ip, ec);
 	if (ec)
 	{
 		std::fprintf(stderr, "ERROR RESOLVING %s: %s\n", destination_ip, ec.message().c_str());
@@ -1059,7 +1151,7 @@ int main(int argc, char* argv[])
 #if !defined __APPLE__
 	// apparently darwin doesn't seems to let you bind to
 	// loopback on any other IP than 127.0.0.1
-	std::uint32_t const ip = addr.to_ulong();
+	std::uint32_t const ip = addr.to_uint();
 	if ((ip & 0xff000000) == 0x7f000000)
 	{
 		local_bind = true;
@@ -1074,9 +1166,9 @@ int main(int argc, char* argv[])
 	}
 
 	std::vector<peer_conn*> conns;
-	conns.reserve(num_connections);
+	conns.reserve(std::size_t(num_connections));
 	int const num_threads = 2;
-	io_service ios[num_threads];
+	io_context ios[num_threads];
 	for (int i = 0; i < num_connections; ++i)
 	{
 		bool corrupt = test_corruption && (i & 1) == 0;
@@ -1084,14 +1176,9 @@ int main(int argc, char* argv[])
 		if (test_mode == upload_test) seed = true;
 		else if (test_mode == dual_test) seed = (i & 1);
 		conns.push_back(new peer_conn(ios[i % num_threads], ti.num_pieces(), ti.piece_length() / 16 / 1024
-			, ep, (char const*)&ti.info_hash()[0], seed, churn, corrupt));
+			, ep, ti.info_hash().data(), seed, churn, corrupt));
 		std::this_thread::sleep_for(std::chrono::milliseconds(1));
-		ios[i % num_threads].poll_one(ec);
-		if (ec)
-		{
-			std::fprintf(stderr, "ERROR: %s\n", ec.message().c_str());
-			break;
-		}
+		ios[i % num_threads].poll_one();
 	}
 
 	std::thread t1(&io_thread, &ios[0]);
@@ -1100,20 +1187,19 @@ int main(int argc, char* argv[])
 	t1.join();
 	t2.join();
 
-	float up = 0.f;
-	float down = 0.f;
-	std::uint64_t total_sent = 0;
-	std::uint64_t total_received = 0;
+	double up = 0.0;
+	double down = 0.0;
+	std::int64_t total_sent = 0;
+	std::int64_t total_received = 0;
 
-	for (std::vector<peer_conn*>::iterator i = conns.begin()
-		, end(conns.end()); i != end; ++i)
+	for (peer_conn* p : conns)
 	{
-		peer_conn* p = *i;
 		int time = int(total_milliseconds(p->end_time - p->start_time));
 		if (time == 0) time = 1;
 		total_sent += p->blocks_sent;
-		up += (std::int64_t(p->blocks_sent) * 0x4000) / time / 1000.f;
-		down += (std::int64_t(p->blocks_received) * 0x4000) / time / 1000.f;
+		total_received += p->blocks_received;
+		up += (std::int64_t(p->blocks_sent) * 0x4000) / time / 1000.0;
+		down += (std::int64_t(p->blocks_received) * 0x4000) / time / 1000.0;
 		delete p;
 	}
 
@@ -1122,8 +1208,8 @@ int main(int argc, char* argv[])
 		"total sent: %.1f %% received: %.1f %%\n"
 		"rate sent: %.1f MB/s received: %.1f MB/s\n"
 		, int(num_suggest), int(num_suggested_requests)
-		, total_sent * 0x4000 * 100.f / float(ti.total_size())
-		, total_received * 0x4000 * 100.f / float(ti.total_size())
+		, total_sent * 0x4000 * 100.0 / double(ti.total_size())
+		, total_received * 0x4000 * 100.0 / double(ti.total_size())
 		, up, down);
 
 	return 0;
