@@ -298,6 +298,20 @@ namespace {
 			;
 	}
 
+	int file_perms(open_mode_t const mode)
+	{
+		// rely on default umask to filter x and w permissions
+		// for group and others
+		int permissions = S_IRUSR | S_IWUSR
+			| S_IRGRP | S_IWGRP
+			| S_IROTH | S_IWOTH;
+
+		if ((mode & aux::open_mode::executable))
+			permissions |= S_IXGRP | S_IXOTH | S_IXUSR;
+
+		return permissions;
+	}
+
 	int mmap_prot(open_mode_t const m)
 	{
 		return (m & open_mode::write)
@@ -323,7 +337,7 @@ namespace {
 
 file_handle::file_handle(string_view name, std::int64_t const size
 	, open_mode_t const mode)
-	: m_fd(open(name.to_string().c_str(), file_flags(mode), 0755))
+	: m_fd(open(name.to_string().c_str(), file_flags(mode), file_perms(mode)))
 {
 #ifdef O_NOATIME
 	if (m_fd < 0 && (mode & open_mode::no_atime))
@@ -331,7 +345,7 @@ file_handle::file_handle(string_view name, std::int64_t const size
 		// NOATIME may not be allowed for certain files, it's best-effort,
 		// so just try again without NOATIME
 		m_fd = open(name.to_string().c_str()
-			, file_flags(mode & ~open_mode::no_atime), 0755);
+			, file_flags(mode & ~open_mode::no_atime), file_perms(mode));
 	}
 #endif
 	if (m_fd < 0) throw_ex<storage_error>(error_code(errno, system_category()), operation_t::file_open);
