@@ -435,6 +435,19 @@ TORRENT_EXPORT std::unique_ptr<disk_interface> mmap_disk_io_constructor(
 		DLOG("destructing mmap_disk_io\n");
 		TORRENT_ASSERT(m_magic == 0x1337);
 		m_magic = 0xdead;
+
+		TORRENT_ASSERT(m_generic_threads.num_threads() == 0);
+		TORRENT_ASSERT(m_hash_threads.num_threads() == 0);
+		if (!m_generic_io_jobs.m_queued_jobs.empty())
+		{
+			for (auto i = m_generic_io_jobs.m_queued_jobs.iterate(); i.get(); i.next())
+				std::printf("generic job: %d\n", int(i.get()->action));
+		}
+		if (!m_hash_io_jobs.m_queued_jobs.empty())
+		{
+			for (auto i = m_hash_io_jobs.m_queued_jobs.iterate(); i.get(); i.next())
+				std::printf("hash job: %d\n", int(i.get()->action));
+		}
 		TORRENT_ASSERT(m_generic_io_jobs.m_queued_jobs.empty());
 		TORRENT_ASSERT(m_hash_io_jobs.m_queued_jobs.empty());
 	}
@@ -1472,7 +1485,8 @@ TORRENT_EXPORT std::unique_ptr<disk_interface> mmap_disk_io_constructor(
 					return true;
 				}
 
-				jobq.m_job_cond.wait(l);
+				using namespace std::literals::chrono_literals;
+				jobq.m_job_cond.wait_for(l, 1s);
 			} while (jobq.m_queued_jobs.empty());
 
 			threads.thread_active();
@@ -1626,7 +1640,8 @@ TORRENT_EXPORT std::unique_ptr<disk_interface> mmap_disk_io_constructor(
 
 	mmap_disk_io::job_queue& mmap_disk_io::queue_for_job(aux::disk_io_job* j)
 	{
-		if (m_hash_threads.max_threads() > 0 && j->action == aux::job_action_t::hash)
+		if (m_hash_threads.max_threads() > 0
+			&& (j->action == aux::job_action_t::hash || j->action == aux::job_action_t::hash2))
 			return m_hash_io_jobs;
 		else
 			return m_generic_io_jobs;
@@ -1634,7 +1649,8 @@ TORRENT_EXPORT std::unique_ptr<disk_interface> mmap_disk_io_constructor(
 
 	aux::disk_io_thread_pool& mmap_disk_io::pool_for_job(aux::disk_io_job* j)
 	{
-		if (m_hash_threads.max_threads() > 0 && j->action == aux::job_action_t::hash)
+		if (m_hash_threads.max_threads() > 0
+			&& (j->action == aux::job_action_t::hash || j->action == aux::job_action_t::hash2))
 			return m_hash_threads;
 		else
 			return m_generic_threads;
