@@ -399,7 +399,10 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		params.info_hash = params.info_hashes.get_best();
+		if (params.info_hashes.v1.is_all_zeros())
+			params.info_hashes.v1 = params.info_hash;
+		if (params.info_hash.is_all_zeros())
+			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
@@ -428,7 +431,10 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		params.info_hash = params.info_hashes.get_best();
+		if (params.info_hashes.v1.is_all_zeros())
+			params.info_hashes.v1 = params.info_hash;
+		if (params.info_hash.is_all_zeros())
+			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
@@ -459,7 +465,10 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		params.info_hash = params.info_hashes.get_best();
+		if (params.info_hashes.v1.is_all_zeros())
+			params.info_hashes.v1 = params.info_hash;
+		if (params.info_hash.is_all_zeros())
+			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
