@@ -1,7 +1,10 @@
 /*
 
-Copyright (c) 2023, Vladimir Golovnev
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2016-2020, Arvid Norberg
+Copyright (c) 2017-2018, Steven Siloti
+Copyright (c) 2017-2018, Alden Torres
+Copyright (c) 2018, Pavel Pimenov
+Copyright (c) 2019, Mike Tzou
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,11 +38,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/file_storage.hpp"
 #include "libtorrent/aux_/alloca.hpp"
 #include "libtorrent/aux_/path.hpp" // for count_bufs
-#include "libtorrent/part_file.hpp"
 #include "libtorrent/session.hpp" // for session::delete_files
 #include "libtorrent/stat_cache.hpp"
 #include "libtorrent/add_torrent_params.hpp"
 #include "libtorrent/torrent_status.hpp"
+#include "libtorrent/error_code.hpp"
 
 #include <set>
 
@@ -178,6 +181,7 @@ namespace libtorrent { namespace aux {
 
 			int const bytes_transferred = op(file_index, file_offset
 				, tmp_buf.first(tmp_bufs_used), ec);
+			TORRENT_ASSERT(bytes_transferred <= file_bytes_left);
 			if (ec) return -1;
 
 			// advance our position in the iovec array and the file offset.
@@ -194,6 +198,8 @@ namespace libtorrent { namespace aux {
 				{
 					// fill in this information in case the caller wants to treat
 					// a short-read as an error
+					ec.operation = operation_t::file_read;
+					ec.ec = boost::asio::error::eof;
 					ec.file(file_index);
 				}
 				return size - bytes_left;
@@ -203,9 +209,9 @@ namespace libtorrent { namespace aux {
 	}
 
 	std::pair<status_t, std::string> move_storage(file_storage const& f
-		, std::string const& save_path
+		, std::string save_path
 		, std::string const& destination_save_path
-		, part_file* pf
+		, std::function<void(std::string const&, error_code&)> const& move_partfile
 		, move_flags_t const flags, storage_error& ec)
 	{
 		status_t ret = status_t::no_error;
@@ -262,12 +268,6 @@ namespace libtorrent { namespace aux {
 			}
 		}
 
-		if (flags == move_flags_t::reset_save_path)
-			return { status_t::need_full_check, new_save_path };
-
-		if (flags == move_flags_t::reset_save_path_unchecked)
-			return { status_t::no_error, new_save_path };
-
 		// indices of all files we ended up copying. These need to be deleted
 		// later
 		aux::vector<bool, file_index_t> copied_files(std::size_t(f.num_files()), false);
@@ -283,7 +283,8 @@ namespace libtorrent { namespace aux {
 			std::string const old_path = combine_path(save_path, f.file_path(i));
 			std::string const new_path = combine_path(new_save_path, f.file_path(i));
 
-			if (flags == move_flags_t::dont_replace && exists(new_path))
+			error_code ignore;
+			if (flags == move_flags_t::dont_replace && exists(new_path, ignore))
 			{
 				if (ret == status_t::no_error) ret = status_t::need_full_check;
 				continue;
@@ -320,9 +321,9 @@ namespace libtorrent { namespace aux {
 			}
 		}
 
-		if (!e && pf)
+		if (!e && move_partfile)
 		{
-			pf->move_partfile(new_save_path, e);
+			move_partfile(new_save_path, e);
 			if (e)
 			{
 				ec.ec = e;
@@ -387,7 +388,7 @@ namespace libtorrent { namespace aux {
 			error_code err;
 			std::string subdir = combine_path(save_path, s);
 
-			while (!compare_path(subdir, save_path) && !err)
+			while (!path_equal(subdir, save_path) && !err)
 			{
 				remove(subdir, err);
 				subdir = parent_path(subdir);
@@ -412,7 +413,7 @@ namespace libtorrent { namespace aux {
 	void delete_files(file_storage const& fs, std::string const& save_path
 		, std::string const& part_file_name, remove_flags_t const options, storage_error& ec)
 	{
-		if (options & session::delete_files)
+		if (options == session::delete_files)
 		{
 			// delete the files from disk
 			std::set<std::string> directories;
@@ -454,9 +455,8 @@ namespace libtorrent { namespace aux {
 			}
 		}
 
-		// when we're deleting "files", we also delete the part file
-		if ((options & session::delete_partfile)
-			|| (options & session::delete_files))
+		if (options == session::delete_files
+			|| options == session::delete_partfile)
 		{
 			error_code error;
 			remove(combine_path(save_path, part_file_name), error);
@@ -603,7 +603,8 @@ std::int64_t get_filesize(stat_cache& stat, file_index_t const file_index
 
 		// parse have bitmask. Verify that the files we expect to have
 		// actually do exist
-		for (piece_index_t i(0); i < rd.have_pieces.end_index(); ++i)
+		piece_index_t const end_piece = std::min(rd.have_pieces.end_index(), fs.end_piece());
+		for (piece_index_t i(0); i < end_piece; ++i)
 		{
 			if (rd.have_pieces.get_bit(i) == false) continue;
 
@@ -663,4 +664,15 @@ std::int64_t get_filesize(stat_cache& stat, file_index_t const file_index
 		return false;
 	}
 
+	int read_zeroes(span<iovec_t const> bufs)
+	{
+		int ret = 0;
+		for (auto buf : bufs)
+		{
+			ret += static_cast<int>(buf.size());
+			std::fill(buf.begin(), buf.end(), '\0');
+		}
+		return ret;
+	}
+
 }}
