@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2017, Arvid Norberg
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2017-2020, Arvid Norberg
+Copyright (c) 2017-2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -42,39 +44,15 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/numeric_cast.hpp" // for clamp
 
 namespace libtorrent {
-namespace {
-	entry build_tracker_list(std::vector<std::string> const& trackers
-		, std::vector<int> const& tracker_tiers)
-	{
-		entry ret;
-		entry::list_type& tr_list = ret.list();
-		if (trackers.empty()) return ret;
-
-		tr_list.emplace_back(entry::list_type());
-		std::size_t tier = 0;
-		auto tier_it = tracker_tiers.begin();
-		for (std::string const& tr : trackers)
-		{
-			if (tier_it != tracker_tiers.end())
-				tier = aux::clamp(std::size_t(*tier_it++), std::size_t{0}, std::size_t{1024});
-
-			if (tr_list.size() <= tier)
-				tr_list.resize(tier + 1);
-
-			tr_list[tier].list().emplace_back(tr);
-		}
-		return ret;
-	}
-}
 
 	entry write_resume_data(add_torrent_params const& atp)
 	{
 		entry ret;
 
-		using namespace libtorrent::detail; // for write_*_endpoint()
+		using namespace libtorrent::aux; // for write_*_endpoint()
 		ret["file-format"] = "libtorrent resume file";
 		ret["file-version"] = 1;
-		ret["libtorrent-version"] = LIBTORRENT_VERSION;
+		ret["libtorrent-version"] = lt::version_str;
 		ret["allocation"] = atp.storage_mode == storage_mode_allocate
 			? "allocate" : "sparse";
 
@@ -120,16 +98,15 @@ namespace {
 #if TORRENT_ABI_VERSION == 1
 		// deprecated in 1.2
 		if (!atp.url.empty()) ret["url"] = atp.url;
-		if (!atp.uuid.empty()) ret["uuid"] = atp.uuid;
 #endif
 
-		ret["info-hash"] = atp.info_hash;
+		ret["info-hash"] = atp.info_hashes.v1;
+		ret["info-hash2"] = atp.info_hashes.v2;
 
 		if (atp.ti)
 		{
-			auto const info = atp.ti->metadata();
-			int const size = atp.ti->metadata_size();
-			ret["info"].preformatted().assign(&info[0], &info[0] + size);
+			auto const info = atp.ti->info_section();
+			ret["info"].preformatted().assign(info.data(), info.data() + info.size());
 			if (!atp.ti->comment().empty())
 				ret["comment"] = atp.ti->comment();
 			if (atp.ti->creation_date() != 0)
@@ -138,14 +115,46 @@ namespace {
 				ret["created by"] = atp.ti->creator();
 		}
 
-		if (!atp.merkle_tree.empty())
+		if (!atp.merkle_trees.empty())
 		{
-			// we need to save the whole merkle hash tree
-			// in order to resume
-			std::string& tree_str = ret["merkle tree"].string();
-			auto const& tree = atp.merkle_tree;
-			tree_str.resize(tree.size() * 20);
-			std::memcpy(&tree_str[0], &tree[0], tree.size() * 20);
+			auto& trees = atp.merkle_trees;
+			auto& ret_trees = ret["trees"].list();
+			ret_trees.reserve(atp.merkle_trees.size());
+			for (file_index_t f(0); f < file_index_t{int(atp.merkle_trees.size())}; ++f)
+			{
+				auto const& tree = trees[f];
+				ret_trees.emplace_back(entry::dictionary_t);
+				auto& ret_dict = ret_trees.back().dict();
+				auto& ret_tree = ret_dict["hashes"].string();
+
+				ret_tree.reserve(tree.size() * 32);
+				for (auto const& n : tree)
+					ret_tree.append(n.data(), n.size());
+
+				if (f < atp.verified_leaf_hashes.end_index())
+				{
+					auto const& verified = atp.verified_leaf_hashes[f];
+					if (!verified.empty())
+					{
+						auto& ret_verified = ret_dict["verified"].string();
+						ret_verified.reserve(verified.size());
+						for (auto const bit : verified)
+							ret_verified.push_back(bit ? '1' : '0');
+					}
+				}
+
+				if (f < atp.merkle_tree_mask.end_index())
+				{
+					auto const& mask = atp.merkle_tree_mask[f];
+					if (!mask.empty())
+					{
+						auto& ret_mask = ret_dict["mask"].string();
+						ret_mask.reserve(mask.size());
+						for (auto const bit : mask)
+							ret_mask.push_back(bit ? '1' : '0');
+					}
+				}
+			}
 		}
 
 		if (!atp.unfinished_pieces.empty())
@@ -217,7 +226,7 @@ namespace {
 			entry::list_type& fl = ret["mapped_files"].list();
 			for (auto const& ent : atp.renamed_files)
 			{
-				std::size_t const idx(static_cast<std::size_t>(static_cast<int>(ent.first)));
+				auto const idx = static_cast<std::size_t>(static_cast<int>(ent.first));
 				if (idx >= fl.size()) fl.resize(idx + 1);
 				fl[idx] = ent.second;
 			}
@@ -283,87 +292,4 @@ namespace {
 		bencode(std::back_inserter(ret), rd);
 		return ret;
 	}
-
-	entry write_torrent_file(add_torrent_params const& atp)
-	{
-		return write_torrent_file(atp, {});
-	}
-
-	entry write_torrent_file(add_torrent_params const& atp, write_torrent_flags_t const flags)
-	{
-		entry ret;
-		if (!atp.ti)
-			aux::throw_ex<system_error>(errors::torrent_missing_info);
-
-		auto const info = span<char const>(atp.ti->metadata().get(), atp.ti->metadata_size());
-		ret["info"].preformatted().assign(info.data(), info.data() + info.size());
-		if (!atp.ti->comment().empty())
-			ret["comment"] = atp.ti->comment();
-		if (atp.ti->creation_date() != 0)
-			ret["creation date"] = atp.ti->creation_date();
-		if (!atp.ti->creator().empty())
-			ret["created by"] = atp.ti->creator();
-
-		// save web seeds
-		if (!atp.url_seeds.empty() && !(flags & write_flags::no_http_seeds))
-		{
-			auto& url_list = ret["url-list"].list();
-			url_list.reserve(atp.url_seeds.size());
-			std::copy(atp.url_seeds.begin(), atp.url_seeds.end(), std::back_inserter(url_list));
-		}
-
-		if (!atp.http_seeds.empty() && !(flags & write_flags::no_http_seeds))
-		{
-			auto& httpseeds_list = ret["httpseeds"].list();
-			httpseeds_list.reserve(atp.http_seeds.size());
-			std::copy(atp.http_seeds.begin(), atp.http_seeds.end(), std::back_inserter(httpseeds_list));
-		}
-
-		// save DHT nodes
-		if (!atp.dht_nodes.empty() && (flags & write_flags::include_dht_nodes))
-		{
-			auto& nodes = ret["nodes"].list();
-			nodes.reserve(atp.dht_nodes.size());
-			for (auto const& n : atp.dht_nodes)
-			{
-				entry::list_type node(2);
-				node[0] = std::move(n.first);
-				node[1] = n.second;
-				nodes.emplace_back(std::move(node));
-			}
-		}
-
-		if (!atp.ti->similar_torrents().empty() && !atp.ti->info("similar"))
-		{
-			auto& l = ret["similar"].list();
-			l.reserve(atp.ti->similar_torrents().size());
-			for (auto const& n : atp.ti->similar_torrents())
-				l.emplace_back(n.to_string());
-		}
-
-		if (!atp.ti->collections().empty() && !atp.ti->info("collections"))
-		{
-			auto& l = ret["collections"].list();
-			l.reserve(atp.ti->collections().size());
-			for (auto const& n : atp.ti->collections())
-				l.emplace_back(n);
-		}
-
-		// save trackers
-		if (atp.trackers.size() == 1)
-			ret["announce"] = atp.trackers.front();
-		else if (atp.trackers.size() > 1)
-			ret["announce-list"] = build_tracker_list(atp.trackers, atp.tracker_tiers);
-
-		return ret;
-	}
-
-	std::vector<char> write_torrent_file_buf(add_torrent_params const& atp
-		, write_torrent_flags_t flags)
-	{
-		std::vector<char> ret;
-		entry e = write_torrent_file(atp, flags);
-		bencode(std::back_inserter(ret), e);
-		return ret;
-	}
 }
