@@ -714,6 +714,23 @@ std::vector<std::string> get_python()
 	return ret;
 }
 
+int find_available_port()
+{
+	int port = 2000 + (::getpid() + _g_test_idx + std::rand()) % 60000;
+	error_code ec;
+	io_context ios;
+
+	// make sure the port we pick is free
+	do {
+		++port;
+		tcp::socket s(ios);
+		s.open(tcp::v4(), ec);
+		if (ec) break;
+		s.bind(tcp::endpoint(make_address("127.0.0.1")
+			, std::uint16_t(port)), ec);
+	} while (ec);
+	return port;
+}
 } // anonymous namespace
 
 // returns a port on success and -1 on failure
@@ -727,20 +744,7 @@ int start_proxy(int proxy_type)
 		if (i->second.type == proxy_type) { return i->first; }
 	}
 
-	int port = 10000 + static_cast<int>(std::rand() % 50000);
-	error_code ec;
-	io_context ios;
-
-	// make sure the port we pick is free
-	do {
-		++port;
-		tcp::socket s(ios);
-		s.open(tcp::v4(), ec);
-		if (ec) break;
-		s.bind(tcp::endpoint(make_address("127.0.0.1")
-			, std::uint16_t(port)), ec);
-	} while (ec);
-
+	int const port = find_available_port();
 
 	char const* type = "";
 	char const* auth = "";
@@ -1163,19 +1167,7 @@ pid_type web_server_pid = 0;
 
 int start_web_server(bool ssl, bool chunked_encoding, bool keepalive, int min_interval)
 {
-	int port = 2000 + static_cast<int>(std::rand() % 6000);
-	error_code ec;
-	io_context ios;
-
-	// make sure the port we pick is free
-	do {
-		++port;
-		tcp::socket s(ios);
-		s.open(tcp::v4(), ec);
-		if (ec) break;
-		s.bind(tcp::endpoint(make_address("127.0.0.1")
-			, std::uint16_t(port)), ec);
-	} while (ec);
+	int const port = find_available_port();
 
 	std::vector<std::string> python_exes = get_python();
 
